@@ -15,42 +15,116 @@
 """Model a set of read-only queries to a database as a snapshot."""
 
 import functools
-
+import threading
 from google.protobuf.struct_pb2 import Struct
-from google.cloud.spanner_v1.proto.transaction_pb2 import TransactionOptions
-from google.cloud.spanner_v1.proto.transaction_pb2 import TransactionSelector
+from google.cloud.spanner_v1 import ExecuteSqlRequest
+from google.cloud.spanner_v1 import ReadRequest
+from google.cloud.spanner_v1 import TransactionOptions
+from google.cloud.spanner_v1 import TransactionSelector
+from google.cloud.spanner_v1 import PartitionOptions
+from google.cloud.spanner_v1 import PartitionQueryRequest
+from google.cloud.spanner_v1 import PartitionReadRequest
 
+from google.api_core.exceptions import InternalServerError
 from google.api_core.exceptions import ServiceUnavailable
-import google.api_core.gapic_v1.method
-from google.cloud._helpers import _datetime_to_pb_timestamp
-from google.cloud.spanner_v1._helpers import _merge_query_options
-from google.cloud._helpers import _timedelta_to_duration_pb
-from google.cloud.spanner_v1._helpers import _make_value_pb
-from google.cloud.spanner_v1._helpers import _metadata_with_prefix
-from google.cloud.spanner_v1._helpers import _SessionWrapper
+from google.api_core.exceptions import InvalidArgument
+from google.api_core import gapic_v1
+from google.cloud.spanner_v1._helpers import (
+    _make_value_pb,
+    _merge_query_options,
+    _metadata_with_prefix,
+    _metadata_with_leader_aware_routing,
+    _retry,
+    _check_rst_stream_error,
+    _SessionWrapper,
+)
+from google.cloud.spanner_v1._opentelemetry_tracing import trace_call
 from google.cloud.spanner_v1.streamed import StreamedResultSet
-from google.cloud.spanner_v1.types import PartitionOptions
+from google.cloud.spanner_v1 import RequestOptions
+
+_STREAM_RESUMPTION_INTERNAL_ERROR_MESSAGES = (
+    "RST_STREAM",
+    "Received unexpected EOS on DATA frame from server",
+)
 
 
-def _restart_on_unavailable(restart):
+def _restart_on_unavailable(
+    method,
+    request,
+    trace_name=None,
+    session=None,
+    attributes=None,
+    transaction=None,
+    transaction_selector=None,
+):
     """Restart iteration after :exc:`.ServiceUnavailable`.
 
-    :type restart: callable
-    :param restart: curried function returning iterator
+    :type method: callable
+    :param method: function returning iterator
+
+    :type request: proto
+    :param request: request proto to call the method with
+
+    :type transaction: :class:`google.cloud.spanner_v1.snapshot._SnapshotBase`
+    :param transaction: Snapshot or Transaction class object based on the type of transaction
+
+    :type transaction_selector: :class:`transaction_pb2.TransactionSelector`
+    :param transaction_selector: Transaction selector object to be used in request if transaction is not passed,
+    if both transaction_selector and transaction are passed, then transaction is given priority.
     """
+
     resume_token = b""
     item_buffer = []
-    iterator = restart()
+
+    if transaction is not None:
+        transaction_selector = transaction._make_txn_selector()
+    elif transaction_selector is None:
+        raise InvalidArgument(
+            "Either transaction or transaction_selector should be set"
+        )
+
+    request.transaction = transaction_selector
+    with trace_call(trace_name, session, attributes):
+        iterator = method(request=request)
     while True:
         try:
             for item in iterator:
                 item_buffer.append(item)
+                # Setting the transaction id because the transaction begin was inlined for first rpc.
+                if (
+                    transaction is not None
+                    and transaction._transaction_id is None
+                    and item.metadata is not None
+                    and item.metadata.transaction is not None
+                    and item.metadata.transaction.id is not None
+                ):
+                    transaction._transaction_id = item.metadata.transaction.id
                 if item.resume_token:
                     resume_token = item.resume_token
                     break
         except ServiceUnavailable:
             del item_buffer[:]
-            iterator = restart(resume_token=resume_token)
+            with trace_call(trace_name, session, attributes):
+                request.resume_token = resume_token
+                if transaction is not None:
+                    transaction_selector = transaction._make_txn_selector()
+                request.transaction = transaction_selector
+                iterator = method(request=request)
+            continue
+        except InternalServerError as exc:
+            resumable_error = any(
+                resumable_message in exc.message
+                for resumable_message in _STREAM_RESUMPTION_INTERNAL_ERROR_MESSAGES
+            )
+            if not resumable_error:
+                raise
+            del item_buffer[:]
+            with trace_call(trace_name, session, attributes):
+                request.resume_token = resume_token
+                if transaction is not None:
+                    transaction_selector = transaction._make_txn_selector()
+                request.transaction = transaction_selector
+                iterator = method(request=request)
             continue
 
         if len(item_buffer) == 0:
@@ -72,11 +146,13 @@ class _SnapshotBase(_SessionWrapper):
     """
 
     _multi_use = False
+    _read_only: bool = True
     _transaction_id = None
     _read_request_count = 0
     _execute_sql_count = 0
+    _lock = threading.Lock()
 
-    def _make_txn_selector(self):  # pylint: disable=redundant-returns-doc
+    def _make_txn_selector(self):
         """Helper for :meth:`read` / :meth:`execute_sql`.
 
         Subclasses must override, returning an instance of
@@ -87,7 +163,20 @@ class _SnapshotBase(_SessionWrapper):
         """
         raise NotImplementedError
 
-    def read(self, table, columns, keyset, index="", limit=0, partition=None):
+    def read(
+        self,
+        table,
+        columns,
+        keyset,
+        index="",
+        limit=0,
+        partition=None,
+        request_options=None,
+        data_boost_enabled=False,
+        *,
+        retry=gapic_v1.method.DEFAULT,
+        timeout=gapic_v1.method.DEFAULT,
+    ):
         """Perform a ``StreamingRead`` API request for rows in a table.
 
         :type table: str
@@ -112,6 +201,29 @@ class _SnapshotBase(_SessionWrapper):
                           from :meth:`partition_read`.  Incompatible with
                           ``limit``.
 
+        :type request_options:
+            :class:`google.cloud.spanner_v1.types.RequestOptions`
+        :param request_options:
+                (Optional) Common options for this request.
+                If a dict is provided, it must be of the same form as the protobuf
+                message :class:`~google.cloud.spanner_v1.types.RequestOptions`.
+                Please note, the `transactionTag` setting will be ignored for
+                snapshot as it's not supported for read-only transactions.
+
+        :type retry: :class:`~google.api_core.retry.Retry`
+        :param retry: (Optional) The retry settings for this request.
+
+        :type timeout: float
+        :param timeout: (Optional) The timeout for this request.
+
+        :type data_boost_enabled:
+        :param data_boost_enabled:
+                (Optional) If this is for a partitioned read and this field is
+                set ``true``, the request will be executed via offline access.
+                If the field is set to ``true`` but the request does not set
+                ``partition_token``, the API will return an
+                ``INVALID_ARGUMENT`` error.
+
         :rtype: :class:`~google.cloud.spanner_v1.streamed.StreamedResultSet`
         :returns: a result set instance which can be used to consume rows.
 
@@ -122,28 +234,74 @@ class _SnapshotBase(_SessionWrapper):
         if self._read_request_count > 0:
             if not self._multi_use:
                 raise ValueError("Cannot re-use single-use snapshot.")
-            if self._transaction_id is None:
+            if self._transaction_id is None and self._read_only:
                 raise ValueError("Transaction ID pending.")
 
         database = self._session._database
         api = database.spanner_api
         metadata = _metadata_with_prefix(database.name)
-        transaction = self._make_txn_selector()
+        if not self._read_only and database._route_to_leader_enabled:
+            metadata.append(
+                _metadata_with_leader_aware_routing(database._route_to_leader_enabled)
+            )
 
-        restart = functools.partial(
-            api.streaming_read,
-            self._session.name,
-            table,
-            columns,
-            keyset._to_pb(),
-            transaction=transaction,
+        if request_options is None:
+            request_options = RequestOptions()
+        elif type(request_options) is dict:
+            request_options = RequestOptions(request_options)
+
+        if self._read_only:
+            # Transaction tags are not supported for read only transactions.
+            request_options.transaction_tag = None
+        elif self.transaction_tag is not None:
+            request_options.transaction_tag = self.transaction_tag
+
+        request = ReadRequest(
+            session=self._session.name,
+            table=table,
+            columns=columns,
+            key_set=keyset._to_pb(),
             index=index,
             limit=limit,
             partition_token=partition,
+            request_options=request_options,
+            data_boost_enabled=data_boost_enabled,
+        )
+        restart = functools.partial(
+            api.streaming_read,
+            request=request,
             metadata=metadata,
+            retry=retry,
+            timeout=timeout,
         )
 
-        iterator = _restart_on_unavailable(restart)
+        trace_attributes = {"table_id": table, "columns": columns}
+
+        if self._transaction_id is None:
+            # lock is added to handle the inline begin for first rpc
+            with self._lock:
+                iterator = _restart_on_unavailable(
+                    restart,
+                    request,
+                    "CloudSpanner.ReadOnlyTransaction",
+                    self._session,
+                    trace_attributes,
+                    transaction=self,
+                )
+                self._read_request_count += 1
+                if self._multi_use:
+                    return StreamedResultSet(iterator, source=self)
+                else:
+                    return StreamedResultSet(iterator)
+        else:
+            iterator = _restart_on_unavailable(
+                restart,
+                request,
+                "CloudSpanner.ReadOnlyTransaction",
+                self._session,
+                trace_attributes,
+                transaction=self,
+            )
 
         self._read_request_count += 1
 
@@ -159,9 +317,11 @@ class _SnapshotBase(_SessionWrapper):
         param_types=None,
         query_mode=None,
         query_options=None,
+        request_options=None,
         partition=None,
-        retry=google.api_core.gapic_v1.method.DEFAULT,
-        timeout=google.api_core.gapic_v1.method.DEFAULT,
+        retry=gapic_v1.method.DEFAULT,
+        timeout=gapic_v1.method.DEFAULT,
+        data_boost_enabled=False,
     ):
         """Perform an ``ExecuteStreamingSql`` API request.
 
@@ -178,19 +338,26 @@ class _SnapshotBase(_SessionWrapper):
             required if parameters are passed.
 
         :type query_mode:
-            :class:`~google.cloud.spanner_v1.proto.ExecuteSqlRequest.QueryMode`
+            :class:`~google.cloud.spanner_v1.types.ExecuteSqlRequest.QueryMode`
         :param query_mode: Mode governing return of results / query plan.
             See:
             `QueryMode <https://cloud.google.com/spanner/reference/rpc/google.spanner.v1#google.spanner.v1.ExecuteSqlRequest.QueryMode>`_.
 
         :type query_options:
-            :class:`~google.cloud.spanner_v1.proto.ExecuteSqlRequest.QueryOptions`
+            :class:`~google.cloud.spanner_v1.types.ExecuteSqlRequest.QueryOptions`
                 or :class:`dict`
         :param query_options:
                 (Optional) Query optimizer configuration to use for the given query.
                 If a dict is provided, it must be of the same form as the protobuf
                 message :class:`~google.cloud.spanner_v1.types.QueryOptions`
 
+        :type request_options:
+            :class:`google.cloud.spanner_v1.types.RequestOptions`
+        :param request_options:
+                (Optional) Common options for this request.
+                If a dict is provided, it must be of the same form as the protobuf
+                message :class:`~google.cloud.spanner_v1.types.RequestOptions`.
+
         :type partition: bytes
         :param partition: (Optional) one of the partition tokens returned
                           from :meth:`partition_query`.
@@ -198,6 +365,20 @@ class _SnapshotBase(_SessionWrapper):
         :rtype: :class:`~google.cloud.spanner_v1.streamed.StreamedResultSet`
         :returns: a result set instance which can be used to consume rows.
 
+        :type retry: :class:`~google.api_core.retry.Retry`
+        :param retry: (Optional) The retry settings for this request.
+
+        :type timeout: float
+        :param timeout: (Optional) The timeout for this request.
+
+        :type data_boost_enabled:
+        :param data_boost_enabled:
+                (Optional) If this is for a partitioned query and this field is
+                set ``true``, the request will be executed via offline access.
+                If the field is set to ``true`` but the request does not set
+                ``partition_token``, the API will return an
+                ``INVALID_ARGUMENT`` error.
+
         :raises ValueError:
             for reuse of single-use snapshots, or if a transaction ID is
             already pending for multiple-use snapshots.
@@ -205,7 +386,7 @@ class _SnapshotBase(_SessionWrapper):
         if self._read_request_count > 0:
             if not self._multi_use:
                 raise ValueError("Cannot re-use single-use snapshot.")
-            if self._transaction_id is None:
+            if self._transaction_id is None and self._read_only:
                 raise ValueError("Transaction ID pending.")
 
         if params is not None:
@@ -215,11 +396,15 @@ class _SnapshotBase(_SessionWrapper):
                 fields={key: _make_value_pb(value) for key, value in params.items()}
             )
         else:
-            params_pb = None
+            params_pb = {}
 
         database = self._session._database
         metadata = _metadata_with_prefix(database.name)
-        transaction = self._make_txn_selector()
+        if not self._read_only and database._route_to_leader_enabled:
+            metadata.append(
+                _metadata_with_leader_aware_routing(database._route_to_leader_enabled)
+            )
+
         api = database.spanner_api
 
         # Query-level options have higher precedence than client-level and
@@ -227,23 +412,65 @@ class _SnapshotBase(_SessionWrapper):
         default_query_options = database._instance._client._query_options
         query_options = _merge_query_options(default_query_options, query_options)
 
-        restart = functools.partial(
-            api.execute_streaming_sql,
-            self._session.name,
-            sql,
-            transaction=transaction,
+        if request_options is None:
+            request_options = RequestOptions()
+        elif type(request_options) is dict:
+            request_options = RequestOptions(request_options)
+        if self._read_only:
+            # Transaction tags are not supported for read only transactions.
+            request_options.transaction_tag = None
+        elif self.transaction_tag is not None:
+            request_options.transaction_tag = self.transaction_tag
+
+        request = ExecuteSqlRequest(
+            session=self._session.name,
+            sql=sql,
             params=params_pb,
             param_types=param_types,
             query_mode=query_mode,
             partition_token=partition,
             seqno=self._execute_sql_count,
             query_options=query_options,
+            request_options=request_options,
+            data_boost_enabled=data_boost_enabled,
+        )
+        restart = functools.partial(
+            api.execute_streaming_sql,
+            request=request,
             metadata=metadata,
             retry=retry,
             timeout=timeout,
         )
 
-        iterator = _restart_on_unavailable(restart)
+        trace_attributes = {"db.statement": sql}
+
+        if self._transaction_id is None:
+            # lock is added to handle the inline begin for first rpc
+            with self._lock:
+                iterator = _restart_on_unavailable(
+                    restart,
+                    request,
+                    "CloudSpanner.ReadWriteTransaction",
+                    self._session,
+                    trace_attributes,
+                    transaction=self,
+                )
+                self._read_request_count += 1
+                self._execute_sql_count += 1
+
+                if self._multi_use:
+                    return StreamedResultSet(iterator, source=self)
+                else:
+                    return StreamedResultSet(iterator)
+        else:
+            iterator = _restart_on_unavailable(
+                restart,
+                request,
+                "CloudSpanner.ReadWriteTransaction",
+                self._session,
+                trace_attributes,
+                transaction=self,
+            )
 
         self._read_request_count += 1
         self._execute_sql_count += 1
@@ -261,8 +488,11 @@ class _SnapshotBase(_SessionWrapper):
         index="",
         partition_size_bytes=None,
         max_partitions=None,
+        *,
+        retry=gapic_v1.method.DEFAULT,
+        timeout=gapic_v1.method.DEFAULT,
     ):
-        """Perform a ``ParitionRead`` API request for rows in a table.
+        """Perform a ``PartitionRead`` API request for rows in a table.
 
         :type table: str
         :param table: name of the table from which to fetch data
@@ -288,12 +518,18 @@ class _SnapshotBase(_SessionWrapper):
             service uses this as a hint, the actual number of partitions may
             differ.
 
+        :type retry: :class:`~google.api_core.retry.Retry`
+        :param retry: (Optional) The retry settings for this request.
+
+        :type timeout: float
+        :param timeout: (Optional) The timeout for this request.
+
         :rtype: iterable of bytes
         :returns: a sequence of partition tokens
 
         :raises ValueError:
             for single-use snapshots, or if a transaction ID is
-            already associtated with the snapshot.
+            already associated with the snapshot.
         """
         if not self._multi_use:
             raise ValueError("Cannot use single-use snapshot.")
@@ -304,12 +540,15 @@ class _SnapshotBase(_SessionWrapper):
         database = self._session._database
         api = database.spanner_api
         metadata = _metadata_with_prefix(database.name)
+        if database._route_to_leader_enabled:
+            metadata.append(
+                _metadata_with_leader_aware_routing(database._route_to_leader_enabled)
+            )
         transaction = self._make_txn_selector()
         partition_options = PartitionOptions(
             partition_size_bytes=partition_size_bytes, max_partitions=max_partitions
         )
-
-        response = api.partition_read(
+        request = PartitionReadRequest(
             session=self._session.name,
             table=table,
             columns=columns,
@@ -317,9 +556,24 @@ class _SnapshotBase(_SessionWrapper):
             transaction=transaction,
             index=index,
             partition_options=partition_options,
-            metadata=metadata,
         )
 
+        trace_attributes = {"table_id": table, "columns": columns}
+        with trace_call(
+            "CloudSpanner.PartitionReadOnlyTransaction", self._session, trace_attributes
+        ):
+            method = functools.partial(
+                api.partition_read,
+                request=request,
+                metadata=metadata,
+                retry=retry,
+                timeout=timeout,
+            )
+            response = _retry(
+                method,
+                allowed_exceptions={InternalServerError: _check_rst_stream_error},
+            )
+
         return [partition.partition_token for partition in response.partitions]
 
     def partition_query(
@@ -329,8 +583,11 @@ class _SnapshotBase(_SessionWrapper):
         param_types=None,
         partition_size_bytes=None,
         max_partitions=None,
+        *,
+        retry=gapic_v1.method.DEFAULT,
+        timeout=gapic_v1.method.DEFAULT,
     ):
-        """Perform a ``ParitionQuery`` API request.
+        """Perform a ``PartitionQuery`` API request.
 
         :type sql: str
         :param sql: SQL query statement
@@ -355,12 +612,18 @@ class _SnapshotBase(_SessionWrapper):
             service uses this as a hint, the actual number of partitions may
             differ.
 
+        :type retry: :class:`~google.api_core.retry.Retry`
+        :param retry: (Optional) The retry settings for this request.
+
+        :type timeout: float
+        :param timeout: (Optional) The timeout for this request.
+
         :rtype: iterable of bytes
         :returns: a sequence of partition tokens
 
         :raises ValueError:
             for single-use snapshots, or if a transaction ID is
-            already associtated with the snapshot.
+            already associated with the snapshot.
         """
         if not self._multi_use:
             raise ValueError("Cannot use single-use snapshot.")
@@ -372,29 +635,49 @@ class _SnapshotBase(_SessionWrapper):
             if param_types is None:
                 raise ValueError("Specify 'param_types' when passing 'params'.")
             params_pb = Struct(
-                fields={key: _make_value_pb(value) for key, value in params.items()}
+                fields={key: _make_value_pb(value) for (key, value) in params.items()}
             )
         else:
-            params_pb = None
+            params_pb = Struct()
 
         database = self._session._database
         api = database.spanner_api
         metadata = _metadata_with_prefix(database.name)
+        if database._route_to_leader_enabled:
+            metadata.append(
+                _metadata_with_leader_aware_routing(database._route_to_leader_enabled)
+            )
         transaction = self._make_txn_selector()
         partition_options = PartitionOptions(
             partition_size_bytes=partition_size_bytes, max_partitions=max_partitions
         )
-
-        response = api.partition_query(
+        request = PartitionQueryRequest(
             session=self._session.name,
             sql=sql,
             transaction=transaction,
             params=params_pb,
             param_types=param_types,
             partition_options=partition_options,
-            metadata=metadata,
         )
 
+        trace_attributes = {"db.statement": sql}
+        with trace_call(
+            "CloudSpanner.PartitionReadWriteTransaction",
+            self._session,
+            trace_attributes,
+        ):
+            method = functools.partial(
+                api.partition_query,
+                request=request,
+                metadata=metadata,
+                retry=retry,
+                timeout=timeout,
+            )
+            response = _retry(
+                method,
+                allowed_exceptions={InternalServerError: _check_rst_stream_error},
+            )
+
         return [partition.partition_token for partition in response.partitions]
 
 
@@ -408,7 +691,7 @@ class Snapshot(_SnapshotBase):
     at a timestamp where all previously committed transactions are visible.
 
     :type session: :class:`~google.cloud.spanner_v1.session.Session`
-    :param session: the session used to perform the commit.
+    :param session: The session used to perform the commit.
 
     :type read_timestamp: :class:`datetime.datetime`
     :param read_timestamp: Execute all reads at the given timestamp.
@@ -426,7 +709,7 @@ class Snapshot(_SnapshotBase):
                             ``exact_staleness`` old.
 
     :type multi_use: :class:`bool`
-    :param multi_use: If true, multipl :meth:`read` / :meth:`execute_sql`
+    :param multi_use: If true, multiple :meth:`read` / :meth:`execute_sql`
                       calls can be performed with the snapshot in the
                       context of a read-only transaction, used to ensure
                       isolation / consistency. Incompatible with
@@ -470,16 +753,16 @@ class Snapshot(_SnapshotBase):
 
         if self._read_timestamp:
             key = "read_timestamp"
-            value = _datetime_to_pb_timestamp(self._read_timestamp)
+            value = self._read_timestamp
         elif self._min_read_timestamp:
             key = "min_read_timestamp"
-            value = _datetime_to_pb_timestamp(self._min_read_timestamp)
+            value = self._min_read_timestamp
         elif self._max_staleness:
             key = "max_staleness"
-            value = _timedelta_to_duration_pb(self._max_staleness)
+            value = self._max_staleness
         elif self._exact_staleness:
             key = "exact_staleness"
-            value = _timedelta_to_duration_pb(self._exact_staleness)
+            value = self._exact_staleness
         else:
             key = "strong"
             value = True
@@ -514,9 +797,21 @@ class Snapshot(_SnapshotBase):
         database = self._session._database
         api = database.spanner_api
         metadata = _metadata_with_prefix(database.name)
+        if not self._read_only and database._route_to_leader_enabled:
+            metadata.append(
+                (_metadata_with_leader_aware_routing(database._route_to_leader_enabled))
+            )
         txn_selector = self._make_txn_selector()
-        response = api.begin_transaction(
-            self._session.name, txn_selector.begin, metadata=metadata
-        )
+        with trace_call("CloudSpanner.BeginTransaction", self._session):
+            method = functools.partial(
+                api.begin_transaction,
+                session=self._session.name,
+                options=txn_selector.begin,
+                metadata=metadata,
+            )
+            response = _retry(
+                method,
+                allowed_exceptions={InternalServerError: _check_rst_stream_error},
+            )
         self._transaction_id = response.id
         return self._transaction_id
