@@ -14,6 +14,7 @@
 
 
 import unittest
+import mock
 
 
 class Test_merge_query_options(unittest.TestCase):
@@ -28,16 +29,22 @@ class Test_merge_query_options(unittest.TestCase):
         self.assertIsNone(result)
 
     def test_base_dict_and_merge_none(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
-        base = {"optimizer_version": "2"}
+        base = {
+            "optimizer_version": "2",
+            "optimizer_statistics_package": "auto_20191128_14_47_22UTC",
+        }
         merge = None
-        expected = ExecuteSqlRequest.QueryOptions(optimizer_version="2")
+        expected = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="2",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         result = self._callFUT(base, merge)
         self.assertEqual(result, expected)
 
     def test_base_empty_and_merge_empty(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         base = ExecuteSqlRequest.QueryOptions()
         merge = ExecuteSqlRequest.QueryOptions()
@@ -45,15 +52,18 @@ class Test_merge_query_options(unittest.TestCase):
         self.assertIsNone(result)
 
     def test_base_none_merge_object(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         base = None
-        merge = ExecuteSqlRequest.QueryOptions(optimizer_version="3")
+        merge = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="3",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         result = self._callFUT(base, merge)
         self.assertEqual(result, merge)
 
     def test_base_none_merge_dict(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         base = None
         merge = {"optimizer_version": "3"}
@@ -62,11 +72,17 @@ class Test_merge_query_options(unittest.TestCase):
         self.assertEqual(result, expected)
 
     def test_base_object_merge_dict(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
-        base = ExecuteSqlRequest.QueryOptions(optimizer_version="1")
+        base = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="1",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         merge = {"optimizer_version": "3"}
-        expected = ExecuteSqlRequest.QueryOptions(optimizer_version="3")
+        expected = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="3",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         result = self._callFUT(base, merge)
         self.assertEqual(result, expected)
 
@@ -98,7 +114,7 @@ class Test_make_value_pb(unittest.TestCase):
     def test_w_explicit_unicode(self):
         from google.protobuf.struct_pb2 import Value
 
-        TEXT = u"TEXT"
+        TEXT = "TEXT"
         value_pb = self._callFUT(TEXT)
         self.assertIsInstance(value_pb, Value)
         self.assertEqual(value_pb.string_value, TEXT)
@@ -107,21 +123,21 @@ class Test_make_value_pb(unittest.TestCase):
         from google.protobuf.struct_pb2 import Value
         from google.protobuf.struct_pb2 import ListValue
 
-        value_pb = self._callFUT([u"a", u"b", u"c"])
+        value_pb = self._callFUT(["a", "b", "c"])
         self.assertIsInstance(value_pb, Value)
         self.assertIsInstance(value_pb.list_value, ListValue)
         values = value_pb.list_value.values
-        self.assertEqual([value.string_value for value in values], [u"a", u"b", u"c"])
+        self.assertEqual([value.string_value for value in values], ["a", "b", "c"])
 
     def test_w_tuple(self):
         from google.protobuf.struct_pb2 import Value
         from google.protobuf.struct_pb2 import ListValue
 
-        value_pb = self._callFUT((u"a", u"b", u"c"))
+        value_pb = self._callFUT(("a", "b", "c"))
         self.assertIsInstance(value_pb, Value)
         self.assertIsInstance(value_pb.list_value, ListValue)
         values = value_pb.list_value.values
-        self.assertEqual([value.string_value for value in values], [u"a", u"b", u"c"])
+        self.assertEqual([value.string_value for value in values], ["a", "b", "c"])
 
     def test_w_bool(self):
         from google.protobuf.struct_pb2 import Value
@@ -131,11 +147,9 @@ class Test_make_value_pb(unittest.TestCase):
         self.assertEqual(value_pb.bool_value, True)
 
     def test_w_int(self):
-        import six
         from google.protobuf.struct_pb2 import Value
 
-        for int_type in six.integer_types:  # include 'long' on Python 2
-            value_pb = self._callFUT(int_type(42))
+        value_pb = self._callFUT(42)
         self.assertIsInstance(value_pb, Value)
         self.assertEqual(value_pb.string_value, "42")
 
@@ -177,12 +191,12 @@ class Test_make_value_pb(unittest.TestCase):
         self.assertEqual(value_pb.string_value, today.isoformat())
 
     def test_w_timestamp_w_nanos(self):
-        import pytz
+        import datetime
         from google.protobuf.struct_pb2 import Value
         from google.api_core import datetime_helpers
 
         when = datetime_helpers.DatetimeWithNanoseconds(
-            2016, 12, 20, 21, 13, 47, nanosecond=123456789, tzinfo=pytz.UTC
+            2016, 12, 20, 21, 13, 47, nanosecond=123456789, tzinfo=datetime.timezone.utc
         )
         value_pb = self._callFUT(when)
         self.assertIsInstance(value_pb, Value)
@@ -199,19 +213,107 @@ class Test_make_value_pb(unittest.TestCase):
 
     def test_w_datetime(self):
         import datetime
-        import pytz
         from google.protobuf.struct_pb2 import Value
         from google.api_core import datetime_helpers
 
-        now = datetime.datetime.utcnow().replace(tzinfo=pytz.UTC)
+        now = datetime.datetime.utcnow().replace(tzinfo=datetime.timezone.utc)
         value_pb = self._callFUT(now)
         self.assertIsInstance(value_pb, Value)
         self.assertEqual(value_pb.string_value, datetime_helpers.to_rfc3339(now))
 
+    def test_w_timestamp_w_tz(self):
+        import datetime
+        from google.protobuf.struct_pb2 import Value
+
+        zone = datetime.timezone(datetime.timedelta(hours=+1), name="CET")
+        when = datetime.datetime(2021, 2, 8, 0, 0, 0, tzinfo=zone)
+        value_pb = self._callFUT(when)
+        self.assertIsInstance(value_pb, Value)
+        self.assertEqual(value_pb.string_value, "2021-02-07T23:00:00.000000Z")
+
     def test_w_unknown_type(self):
         with self.assertRaises(ValueError):
             self._callFUT(object())
 
+    def test_w_numeric_precision_and_scale_valid(self):
+        import decimal
+        from google.protobuf.struct_pb2 import Value
+
+        cases = [
+            decimal.Decimal("42"),
+            decimal.Decimal("9.9999999999999999999999999999999999999E+28"),
+            decimal.Decimal("-9.9999999999999999999999999999999999999E+28"),
+            decimal.Decimal("99999999999999999999999999999.999999999"),
+            decimal.Decimal("1E+28"),
+            decimal.Decimal("1E-9"),
+        ]
+        for value in cases:
+            with self.subTest(value=value):
+                value_pb = self._callFUT(value)
+                self.assertIsInstance(value_pb, Value)
+                self.assertEqual(value_pb.string_value, str(value))
+
+    def test_w_numeric_precision_and_scale_invalid(self):
+        import decimal
+        from google.cloud.spanner_v1._helpers import (
+            NUMERIC_MAX_SCALE_ERR_MSG,
+            NUMERIC_MAX_PRECISION_ERR_MSG,
+        )
+
+        max_precision_error_msg = NUMERIC_MAX_PRECISION_ERR_MSG.format("30")
+        max_scale_error_msg = NUMERIC_MAX_SCALE_ERR_MSG.format("10")
+
+        cases = [
+            (
+                decimal.Decimal("9.9999999999999999999999999999999999999E+29"),
+                max_precision_error_msg,
+            ),
+            (
+                decimal.Decimal("-9.9999999999999999999999999999999999999E+29"),
+                max_precision_error_msg,
+            ),
+            (
+                decimal.Decimal("999999999999999999999999999999.99999999"),
+                max_precision_error_msg,
+            ),
+            (
+                decimal.Decimal("-999999999999999999999999999999.99999999"),
+                max_precision_error_msg,
+            ),
+            (
+                decimal.Decimal("999999999999999999999999999999"),
+                max_precision_error_msg,
+            ),
+            (decimal.Decimal("1E+29"), max_precision_error_msg),
+            (decimal.Decimal("1E-10"), max_scale_error_msg),
+        ]
+
+        for value, err_msg in cases:
+            with self.subTest(value=value, err_msg=err_msg):
+                self.assertRaisesRegex(
+                    ValueError,
+                    err_msg,
+                    lambda: self._callFUT(value),
+                )
+
+    def test_w_json(self):
+        import json
+        from google.protobuf.struct_pb2 import Value
+
+        value = json.dumps(
+            {"id": 27863, "Name": "Anamika"}, sort_keys=True, separators=(",", ":")
+        )
+        value_pb = self._callFUT(value)
+        self.assertIsInstance(value_pb, Value)
+        self.assertEqual(value_pb.string_value, value)
+
+    def test_w_json_None(self):
+        from google.cloud.spanner_v1 import JsonObject
+
+        value = JsonObject(None)
+        value_pb = self._callFUT(value)
+        self.assertTrue(value_pb.HasField("null_value"))
+
 
 class Test_make_list_value_pb(unittest.TestCase):
     def _callFUT(self, *args, **kw):
@@ -229,7 +331,7 @@ class Test_make_list_value_pb(unittest.TestCase):
     def test_w_single_value(self):
         from google.protobuf.struct_pb2 import ListValue
 
-        VALUE = u"value"
+        VALUE = "value"
         result = self._callFUT(values=[VALUE])
         self.assertIsInstance(result, ListValue)
         self.assertEqual(len(result.values), 1)
@@ -238,7 +340,7 @@ class Test_make_list_value_pb(unittest.TestCase):
     def test_w_multiple_values(self):
         from google.protobuf.struct_pb2 import ListValue
 
-        VALUE_1 = u"value"
+        VALUE_1 = "value"
         VALUE_2 = 42
         result = self._callFUT(values=[VALUE_1, VALUE_2])
         self.assertIsInstance(result, ListValue)
@@ -271,7 +373,7 @@ class Test_make_list_value_pbs(unittest.TestCase):
     def test_w_multiple_values(self):
         from google.protobuf.struct_pb2 import ListValue
 
-        values = [[0, u"A"], [1, u"B"]]
+        values = [[0, "A"], [1, "B"]]
         result = self._callFUT(values=values)
         self.assertEqual(len(result), len(values))
         for found, expected in zip(result, values):
@@ -289,85 +391,104 @@ class Test_parse_value_pb(unittest.TestCase):
 
     def test_w_null(self):
         from google.protobuf.struct_pb2 import Value, NULL_VALUE
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, STRING
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
-        field_type = Type(code=STRING)
+        field_type = Type(code=TypeCode.STRING)
         value_pb = Value(null_value=NULL_VALUE)
 
         self.assertEqual(self._callFUT(value_pb, field_type), None)
 
     def test_w_string(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, STRING
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
-        VALUE = u"Value"
-        field_type = Type(code=STRING)
+        VALUE = "Value"
+        field_type = Type(code=TypeCode.STRING)
         value_pb = Value(string_value=VALUE)
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
     def test_w_bytes(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, BYTES
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         VALUE = b"Value"
-        field_type = Type(code=BYTES)
+        field_type = Type(code=TypeCode.BYTES)
         value_pb = Value(string_value=VALUE)
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
     def test_w_bool(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, BOOL
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         VALUE = True
-        field_type = Type(code=BOOL)
+        field_type = Type(code=TypeCode.BOOL)
         value_pb = Value(bool_value=VALUE)
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
     def test_w_int(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, INT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         VALUE = 12345
-        field_type = Type(code=INT64)
+        field_type = Type(code=TypeCode.INT64)
         value_pb = Value(string_value=str(VALUE))
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
     def test_w_float(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, FLOAT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         VALUE = 3.14159
-        field_type = Type(code=FLOAT64)
+        field_type = Type(code=TypeCode.FLOAT64)
         value_pb = Value(number_value=VALUE)
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
+    def test_w_float_str(self):
+        from google.protobuf.struct_pb2 import Value
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
+
+        VALUE = "3.14159"
+        field_type = Type(code=TypeCode.FLOAT64)
+        value_pb = Value(string_value=VALUE)
+        expected_value = 3.14159
+
+        self.assertEqual(self._callFUT(value_pb, field_type), expected_value)
+
     def test_w_date(self):
         import datetime
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, DATE
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         VALUE = datetime.date.today()
-        field_type = Type(code=DATE)
+        field_type = Type(code=TypeCode.DATE)
         value_pb = Value(string_value=VALUE.isoformat())
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
 
     def test_w_timestamp_wo_nanos(self):
-        import pytz
+        import datetime
         from google.protobuf.struct_pb2 import Value
         from google.api_core import datetime_helpers
-        from google.cloud.spanner_v1.proto.type_pb2 import TIMESTAMP
-        from google.cloud.spanner_v1.proto.type_pb2 import Type
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         value = datetime_helpers.DatetimeWithNanoseconds(
-            2016, 12, 20, 21, 13, 47, microsecond=123456, tzinfo=pytz.UTC
+            2016, 12, 20, 21, 13, 47, microsecond=123456, tzinfo=datetime.timezone.utc
         )
-        field_type = Type(code=TIMESTAMP)
+        field_type = Type(code=TypeCode.TIMESTAMP)
         value_pb = Value(string_value=datetime_helpers.to_rfc3339(value))
 
         parsed = self._callFUT(value_pb, field_type)
@@ -375,16 +496,16 @@ class Test_parse_value_pb(unittest.TestCase):
         self.assertEqual(parsed, value)
 
     def test_w_timestamp_w_nanos(self):
-        import pytz
+        import datetime
         from google.protobuf.struct_pb2 import Value
         from google.api_core import datetime_helpers
-        from google.cloud.spanner_v1.proto.type_pb2 import TIMESTAMP
-        from google.cloud.spanner_v1.proto.type_pb2 import Type
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
         value = datetime_helpers.DatetimeWithNanoseconds(
-            2016, 12, 20, 21, 13, 47, nanosecond=123456789, tzinfo=pytz.UTC
+            2016, 12, 20, 21, 13, 47, nanosecond=123456789, tzinfo=datetime.timezone.utc
         )
-        field_type = Type(code=TIMESTAMP)
+        field_type = Type(code=TypeCode.TIMESTAMP)
         value_pb = Value(string_value=datetime_helpers.to_rfc3339(value))
 
         parsed = self._callFUT(value_pb, field_type)
@@ -392,19 +513,25 @@ class Test_parse_value_pb(unittest.TestCase):
         self.assertEqual(parsed, value)
 
     def test_w_array_empty(self):
-        from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, ARRAY, INT64
+        from google.protobuf.struct_pb2 import Value, ListValue
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
-        field_type = Type(code=ARRAY, array_element_type=Type(code=INT64))
-        value_pb = Value()
+        field_type = Type(
+            code=TypeCode.ARRAY, array_element_type=Type(code=TypeCode.INT64)
+        )
+        value_pb = Value(list_value=ListValue(values=[]))
 
         self.assertEqual(self._callFUT(value_pb, field_type), [])
 
     def test_w_array_non_empty(self):
         from google.protobuf.struct_pb2 import Value, ListValue
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, ARRAY, INT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
-        field_type = Type(code=ARRAY, array_element_type=Type(code=INT64))
+        field_type = Type(
+            code=TypeCode.ARRAY, array_element_type=Type(code=TypeCode.INT64)
+        )
         VALUES = [32, 19, 5]
         values_pb = ListValue(
             values=[Value(string_value=str(value)) for value in VALUES]
@@ -415,28 +542,63 @@ class Test_parse_value_pb(unittest.TestCase):
 
     def test_w_struct(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, StructType
-        from google.cloud.spanner_v1.proto.type_pb2 import STRUCT, STRING, INT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import StructType
+        from google.cloud.spanner_v1 import TypeCode
         from google.cloud.spanner_v1._helpers import _make_list_value_pb
 
-        VALUES = [u"phred", 32]
+        VALUES = ["phred", 32]
         struct_type_pb = StructType(
             fields=[
-                StructType.Field(name="name", type=Type(code=STRING)),
-                StructType.Field(name="age", type=Type(code=INT64)),
+                StructType.Field(name="name", type_=Type(code=TypeCode.STRING)),
+                StructType.Field(name="age", type_=Type(code=TypeCode.INT64)),
             ]
         )
-        field_type = Type(code=STRUCT, struct_type=struct_type_pb)
+        field_type = Type(code=TypeCode.STRUCT, struct_type=struct_type_pb)
         value_pb = Value(list_value=_make_list_value_pb(VALUES))
 
         self.assertEqual(self._callFUT(value_pb, field_type), VALUES)
 
+    def test_w_numeric(self):
+        import decimal
+        from google.protobuf.struct_pb2 import Value
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
+
+        VALUE = decimal.Decimal("99999999999999999999999999999.999999999")
+        field_type = Type(code=TypeCode.NUMERIC)
+        value_pb = Value(string_value=str(VALUE))
+
+        self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
+
+    def test_w_json(self):
+        import json
+        from google.protobuf.struct_pb2 import Value
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
+
+        VALUE = {"id": 27863, "Name": "Anamika"}
+        str_repr = json.dumps(VALUE, sort_keys=True, separators=(",", ":"))
+
+        field_type = Type(code=TypeCode.JSON)
+        value_pb = Value(string_value=str_repr)
+
+        self.assertEqual(self._callFUT(value_pb, field_type), VALUE)
+
+        VALUE = None
+        str_repr = json.dumps(VALUE, sort_keys=True, separators=(",", ":"))
+
+        field_type = Type(code=TypeCode.JSON)
+        value_pb = Value(string_value=str_repr)
+
+        self.assertEqual(self._callFUT(value_pb, field_type), {})
+
     def test_w_unknown_type(self):
         from google.protobuf.struct_pb2 import Value
-        from google.cloud.spanner_v1.proto.type_pb2 import Type
-        from google.cloud.spanner_v1.proto.type_pb2 import TYPE_CODE_UNSPECIFIED
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
 
-        field_type = Type(code=TYPE_CODE_UNSPECIFIED)
+        field_type = Type(code=TypeCode.TYPE_CODE_UNSPECIFIED)
         value_pb = Value(string_value="Borked")
 
         with self.assertRaises(ValueError):
@@ -450,28 +612,30 @@ class Test_parse_list_value_pbs(unittest.TestCase):
         return _parse_list_value_pbs(*args, **kw)
 
     def test_empty(self):
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, StructType
-        from google.cloud.spanner_v1.proto.type_pb2 import STRING, INT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import StructType
+        from google.cloud.spanner_v1 import TypeCode
 
         struct_type_pb = StructType(
             fields=[
-                StructType.Field(name="name", type=Type(code=STRING)),
-                StructType.Field(name="age", type=Type(code=INT64)),
+                StructType.Field(name="name", type_=Type(code=TypeCode.STRING)),
+                StructType.Field(name="age", type_=Type(code=TypeCode.INT64)),
             ]
         )
 
         self.assertEqual(self._callFUT(rows=[], row_type=struct_type_pb), [])
 
     def test_non_empty(self):
-        from google.cloud.spanner_v1.proto.type_pb2 import Type, StructType
-        from google.cloud.spanner_v1.proto.type_pb2 import STRING, INT64
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import StructType
+        from google.cloud.spanner_v1 import TypeCode
         from google.cloud.spanner_v1._helpers import _make_list_value_pbs
 
-        VALUES = [[u"phred", 32], [u"bharney", 31]]
+        VALUES = [["phred", 32], ["bharney", 31]]
         struct_type_pb = StructType(
             fields=[
-                StructType.Field(name="name", type=Type(code=STRING)),
-                StructType.Field(name="age", type=Type(code=INT64)),
+                StructType.Field(name="name", type_=Type(code=TypeCode.STRING)),
+                StructType.Field(name="age", type_=Type(code=TypeCode.INT64)),
             ]
         )
         values_pbs = _make_list_value_pbs(VALUES)
@@ -506,3 +670,94 @@ class Test_metadata_with_prefix(unittest.TestCase):
         prefix = "prefix"
         metadata = self._call_fut(prefix)
         self.assertEqual(metadata, [("google-cloud-resource-prefix", prefix)])
+
+
+class Test_retry(unittest.TestCase):
+    class test_class:
+        def test_fxn(self):
+            return True
+
+    def test_retry_on_error(self):
+        from google.api_core.exceptions import InternalServerError, NotFound
+        from google.cloud.spanner_v1._helpers import _retry
+        import functools
+
+        test_api = mock.create_autospec(self.test_class)
+        test_api.test_fxn.side_effect = [
+            InternalServerError("testing"),
+            NotFound("testing"),
+            True,
+        ]
+
+        _retry(functools.partial(test_api.test_fxn))
+
+        self.assertEqual(test_api.test_fxn.call_count, 3)
+
+    def test_retry_allowed_exceptions(self):
+        from google.api_core.exceptions import InternalServerError, NotFound
+        from google.cloud.spanner_v1._helpers import _retry
+        import functools
+
+        test_api = mock.create_autospec(self.test_class)
+        test_api.test_fxn.side_effect = [
+            NotFound("testing"),
+            InternalServerError("testing"),
+            True,
+        ]
+
+        with self.assertRaises(InternalServerError):
+            _retry(
+                functools.partial(test_api.test_fxn),
+                allowed_exceptions={NotFound: None},
+            )
+
+        self.assertEqual(test_api.test_fxn.call_count, 2)
+
+    def test_retry_count(self):
+        from google.api_core.exceptions import InternalServerError
+        from google.cloud.spanner_v1._helpers import _retry
+        import functools
+
+        test_api = mock.create_autospec(self.test_class)
+        test_api.test_fxn.side_effect = [
+            InternalServerError("testing"),
+            InternalServerError("testing"),
+        ]
+
+        with self.assertRaises(InternalServerError):
+            _retry(functools.partial(test_api.test_fxn), retry_count=1)
+
+        self.assertEqual(test_api.test_fxn.call_count, 2)
+
+    def test_check_rst_stream_error(self):
+        from google.api_core.exceptions import InternalServerError
+        from google.cloud.spanner_v1._helpers import _retry, _check_rst_stream_error
+        import functools
+
+        test_api = mock.create_autospec(self.test_class)
+        test_api.test_fxn.side_effect = [
+            InternalServerError("Received unexpected EOS on DATA frame from server"),
+            InternalServerError("RST_STREAM"),
+            True,
+        ]
+
+        _retry(
+            functools.partial(test_api.test_fxn),
+            allowed_exceptions={InternalServerError: _check_rst_stream_error},
+        )
+
+        self.assertEqual(test_api.test_fxn.call_count, 3)
+
+
+class Test_metadata_with_leader_aware_routing(unittest.TestCase):
+    def _call_fut(self, *args, **kw):
+        from google.cloud.spanner_v1._helpers import _metadata_with_leader_aware_routing
+
+        return _metadata_with_leader_aware_routing(*args, **kw)
+
+    def test(self):
+        value = True
+        metadata = self._call_fut(True)
+        self.assertEqual(
+            metadata, ("x-goog-spanner-route-to-leader", str(value).lower())
+        )
