@@ -37,8 +37,10 @@ class TestClient(unittest.TestCase):
     INSTANCE_NAME = "%s/instances/%s" % (PATH, INSTANCE_ID)
     DISPLAY_NAME = "display-name"
     NODE_COUNT = 5
+    PROCESSING_UNITS = 5000
+    LABELS = {"test": "true"}
     TIMEOUT_SECONDS = 80
-    USER_AGENT = "you-sir-age-int"
+    LEADER_OPTIONS = ["leader1", "leader2"]
 
     def _get_target_class(self):
         from google.cloud import spanner
@@ -54,10 +56,10 @@ class TestClient(unittest.TestCase):
         creds,
         expected_creds=None,
         client_info=None,
-        user_agent=None,
         client_options=None,
         query_options=None,
         expected_query_options=None,
+        route_to_leader_enabled=True,
     ):
         import google.api_core.client_options
         from google.cloud.spanner_v1 import client as MUT
@@ -70,17 +72,18 @@ class TestClient(unittest.TestCase):
             expected_client_info = MUT._CLIENT_INFO
 
         kwargs["client_options"] = client_options
-        if type(client_options) == dict:
+        if type(client_options) is dict:
             expected_client_options = google.api_core.client_options.from_dict(
                 client_options
             )
         else:
             expected_client_options = client_options
+        if route_to_leader_enabled is not None:
+            kwargs["route_to_leader_enabled"] = route_to_leader_enabled
 
         client = self._make_one(
             project=self.PROJECT,
             credentials=creds,
-            user_agent=user_agent,
             query_options=query_options,
             **kwargs
         )
@@ -90,11 +93,12 @@ class TestClient(unittest.TestCase):
 
         self.assertIs(client._credentials, expected_creds)
         if expected_scopes is not None:
-            creds.with_scopes.assert_called_once_with(expected_scopes)
+            creds.with_scopes.assert_called_once_with(
+                expected_scopes, default_scopes=None
+            )
 
         self.assertEqual(client.project, self.PROJECT)
         self.assertIs(client._client_info, expected_client_info)
-        self.assertEqual(client.user_agent, user_agent)
         if expected_client_options is not None:
             self.assertIsInstance(
                 client._client_options, google.api_core.client_options.ClientOptions
@@ -105,6 +109,10 @@ class TestClient(unittest.TestCase):
             )
         if expected_query_options is not None:
             self.assertEqual(client._query_options, expected_query_options)
+        if route_to_leader_enabled is not None:
+            self.assertEqual(client.route_to_leader_enabled, route_to_leader_enabled)
+        else:
+            self.assertFalse(client.route_to_leader_enabled)
 
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_emulator_host")
     @mock.patch("warnings.warn")
@@ -127,20 +135,6 @@ class TestClient(unittest.TestCase):
         creds = _make_credentials()
         self._constructor_test_helper(expected_scopes, creds)
 
-    @mock.patch("warnings.warn")
-    def test_constructor_custom_user_agent_and_timeout(self, mock_warn):
-        from google.cloud.spanner_v1 import client as MUT
-
-        CUSTOM_USER_AGENT = "custom-application"
-        expected_scopes = (MUT.SPANNER_ADMIN_SCOPE,)
-        creds = _make_credentials()
-        self._constructor_test_helper(
-            expected_scopes, creds, user_agent=CUSTOM_USER_AGENT
-        )
-        mock_warn.assert_called_once_with(
-            MUT._USER_AGENT_DEPRECATED, DeprecationWarning, stacklevel=2
-        )
-
     def test_constructor_custom_client_info(self):
         from google.cloud.spanner_v1 import client as MUT
 
@@ -150,6 +144,8 @@ class TestClient(unittest.TestCase):
         self._constructor_test_helper(expected_scopes, creds, client_info=client_info)
 
     def test_constructor_implicit_credentials(self):
+        from google.cloud.spanner_v1 import client as MUT
+
         creds = _make_credentials()
 
         patch = mock.patch("google.auth.default", return_value=(creds, None))
@@ -158,7 +154,7 @@ class TestClient(unittest.TestCase):
                 None, None, expected_creds=creds.with_scopes.return_value
             )
 
-        default.assert_called_once_with()
+        default.assert_called_once_with(scopes=(MUT.SPANNER_ADMIN_SCOPE,))
 
     def test_constructor_credentials_wo_create_scoped(self):
         creds = _make_credentials()
@@ -187,46 +183,68 @@ class TestClient(unittest.TestCase):
         )
 
     def test_constructor_custom_query_options_client_config(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
         from google.cloud.spanner_v1 import client as MUT
 
         expected_scopes = (MUT.SPANNER_ADMIN_SCOPE,)
         creds = _make_credentials()
+        query_options = expected_query_options = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="1",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         self._constructor_test_helper(
             expected_scopes,
             creds,
-            query_options=ExecuteSqlRequest.QueryOptions(optimizer_version="1"),
-            expected_query_options=ExecuteSqlRequest.QueryOptions(
-                optimizer_version="1"
-            ),
+            query_options=query_options,
+            expected_query_options=expected_query_options,
         )
 
+    @mock.patch(
+        "google.cloud.spanner_v1.client._get_spanner_optimizer_statistics_package"
+    )
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_optimizer_version")
-    def test_constructor_custom_query_options_env_config(self, mock_ver):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+    def test_constructor_custom_query_options_env_config(self, mock_ver, mock_stats):
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
         from google.cloud.spanner_v1 import client as MUT
 
         expected_scopes = (MUT.SPANNER_ADMIN_SCOPE,)
         creds = _make_credentials()
         mock_ver.return_value = "2"
+        mock_stats.return_value = "auto_20191128_14_47_22UTC"
+        query_options = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="1",
+            optimizer_statistics_package="auto_20191128_10_47_22UTC",
+        )
+        expected_query_options = ExecuteSqlRequest.QueryOptions(
+            optimizer_version="2",
+            optimizer_statistics_package="auto_20191128_14_47_22UTC",
+        )
         self._constructor_test_helper(
             expected_scopes,
             creds,
-            query_options=ExecuteSqlRequest.QueryOptions(optimizer_version="1"),
-            expected_query_options=ExecuteSqlRequest.QueryOptions(
-                optimizer_version="2"
-            ),
+            query_options=query_options,
+            expected_query_options=expected_query_options,
+        )
+
+    def test_constructor_route_to_leader_disbled(self):
+        from google.cloud.spanner_v1 import client as MUT
+
+        expected_scopes = (MUT.SPANNER_ADMIN_SCOPE,)
+        creds = _make_credentials()
+        self._constructor_test_helper(
+            expected_scopes, creds, route_to_leader_enabled=False
         )
 
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_emulator_host")
     def test_instance_admin_api(self, mock_em):
         from google.cloud.spanner_v1.client import SPANNER_ADMIN_SCOPE
+        from google.api_core.client_options import ClientOptions
 
         mock_em.return_value = None
 
         credentials = _make_credentials()
         client_info = mock.Mock()
-        client_options = mock.Mock()
+        client_options = ClientOptions(quota_project_id="QUOTA-PROJECT")
         client = self._make_one(
             project=self.PROJECT,
             credentials=credentials,
@@ -246,19 +264,21 @@ class TestClient(unittest.TestCase):
         self.assertIs(again, api)
 
         instance_admin_client.assert_called_once_with(
-            credentials=credentials.with_scopes.return_value,
-            client_info=client_info,
-            client_options=client_options,
+            credentials=mock.ANY, client_info=client_info, client_options=client_options
         )
 
-        credentials.with_scopes.assert_called_once_with(expected_scopes)
+        credentials.with_scopes.assert_called_once_with(
+            expected_scopes, default_scopes=None
+        )
 
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_emulator_host")
     def test_instance_admin_api_emulator_env(self, mock_em):
+        from google.api_core.client_options import ClientOptions
+
         mock_em.return_value = "emulator.host"
         credentials = _make_credentials()
         client_info = mock.Mock()
-        client_options = mock.Mock()
+        client_options = ClientOptions(api_endpoint="endpoint")
         client = self._make_one(
             project=self.PROJECT,
             credentials=credentials,
@@ -319,11 +339,12 @@ class TestClient(unittest.TestCase):
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_emulator_host")
     def test_database_admin_api(self, mock_em):
         from google.cloud.spanner_v1.client import SPANNER_ADMIN_SCOPE
+        from google.api_core.client_options import ClientOptions
 
         mock_em.return_value = None
         credentials = _make_credentials()
         client_info = mock.Mock()
-        client_options = mock.Mock()
+        client_options = ClientOptions(quota_project_id="QUOTA-PROJECT")
         client = self._make_one(
             project=self.PROJECT,
             credentials=credentials,
@@ -343,19 +364,21 @@ class TestClient(unittest.TestCase):
         self.assertIs(again, api)
 
         database_admin_client.assert_called_once_with(
-            credentials=credentials.with_scopes.return_value,
-            client_info=client_info,
-            client_options=client_options,
+            credentials=mock.ANY, client_info=client_info, client_options=client_options
         )
 
-        credentials.with_scopes.assert_called_once_with(expected_scopes)
+        credentials.with_scopes.assert_called_once_with(
+            expected_scopes, default_scopes=None
+        )
 
     @mock.patch("google.cloud.spanner_v1.client._get_spanner_emulator_host")
     def test_database_admin_api_emulator_env(self, mock_em):
+        from google.api_core.client_options import ClientOptions
+
         mock_em.return_value = "host:port"
         credentials = _make_credentials()
         client_info = mock.Mock()
-        client_options = mock.Mock()
+        client_options = ClientOptions(api_endpoint="endpoint")
         client = self._make_one(
             project=self.PROJECT,
             credentials=credentials,
@@ -436,83 +459,86 @@ class TestClient(unittest.TestCase):
         self.assertEqual(client.project_name, project_name)
 
     def test_list_instance_configs(self):
-        from google.cloud.spanner_admin_instance_v1.gapic import instance_admin_client
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2,
+        from google.cloud.spanner_admin_instance_v1 import InstanceAdminClient
+        from google.cloud.spanner_admin_instance_v1 import (
+            InstanceConfig as InstanceConfigPB,
         )
-        from google.cloud.spanner_v1.client import InstanceConfig
+        from google.cloud.spanner_admin_instance_v1 import ListInstanceConfigsRequest
+        from google.cloud.spanner_admin_instance_v1 import ListInstanceConfigsResponse
 
-        api = instance_admin_client.InstanceAdminClient(mock.Mock())
+        api = InstanceAdminClient(credentials=mock.Mock())
         credentials = _make_credentials()
         client = self._make_one(project=self.PROJECT, credentials=credentials)
         client._instance_admin_api = api
 
-        instance_config_pbs = spanner_instance_admin_pb2.ListInstanceConfigsResponse(
+        instance_config_pbs = ListInstanceConfigsResponse(
             instance_configs=[
-                spanner_instance_admin_pb2.InstanceConfig(
-                    name=self.CONFIGURATION_NAME, display_name=self.DISPLAY_NAME
+                InstanceConfigPB(
+                    name=self.CONFIGURATION_NAME,
+                    display_name=self.DISPLAY_NAME,
+                    leader_options=self.LEADER_OPTIONS,
                 )
             ]
         )
 
-        lic_api = api._inner_api_calls["list_instance_configs"] = mock.Mock(
-            return_value=instance_config_pbs
-        )
+        lic_api = api._transport._wrapped_methods[
+            api._transport.list_instance_configs
+        ] = mock.Mock(return_value=instance_config_pbs)
 
         response = client.list_instance_configs()
         instance_configs = list(response)
 
         instance_config = instance_configs[0]
-        self.assertIsInstance(instance_config, InstanceConfig)
+        self.assertIsInstance(instance_config, InstanceConfigPB)
         self.assertEqual(instance_config.name, self.CONFIGURATION_NAME)
         self.assertEqual(instance_config.display_name, self.DISPLAY_NAME)
+        self.assertEqual(instance_config.leader_options, self.LEADER_OPTIONS)
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", client.project_name),
             ("x-goog-request-params", "parent={}".format(client.project_name)),
-        ]
+        )
         lic_api.assert_called_once_with(
-            spanner_instance_admin_pb2.ListInstanceConfigsRequest(parent=self.PATH),
+            ListInstanceConfigsRequest(parent=self.PATH),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
 
     def test_list_instance_configs_w_options(self):
-        from google.cloud.spanner_admin_instance_v1.gapic import instance_admin_client
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2,
+        from google.cloud.spanner_admin_instance_v1 import InstanceAdminClient
+        from google.cloud.spanner_admin_instance_v1 import (
+            InstanceConfig as InstanceConfigPB,
         )
+        from google.cloud.spanner_admin_instance_v1 import ListInstanceConfigsRequest
+        from google.cloud.spanner_admin_instance_v1 import ListInstanceConfigsResponse
 
-        api = instance_admin_client.InstanceAdminClient(mock.Mock())
+        api = InstanceAdminClient(credentials=mock.Mock())
         credentials = _make_credentials()
         client = self._make_one(project=self.PROJECT, credentials=credentials)
         client._instance_admin_api = api
 
-        instance_config_pbs = spanner_instance_admin_pb2.ListInstanceConfigsResponse(
+        instance_config_pbs = ListInstanceConfigsResponse(
             instance_configs=[
-                spanner_instance_admin_pb2.InstanceConfig(
+                InstanceConfigPB(
                     name=self.CONFIGURATION_NAME, display_name=self.DISPLAY_NAME
                 )
             ]
         )
 
-        lic_api = api._inner_api_calls["list_instance_configs"] = mock.Mock(
-            return_value=instance_config_pbs
-        )
+        lic_api = api._transport._wrapped_methods[
+            api._transport.list_instance_configs
+        ] = mock.Mock(return_value=instance_config_pbs)
 
-        token = "token"
         page_size = 42
-        list(client.list_instance_configs(page_token=token, page_size=42))
+        list(client.list_instance_configs(page_size=42))
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", client.project_name),
             ("x-goog-request-params", "parent={}".format(client.project_name)),
-        ]
+        )
         lic_api.assert_called_once_with(
-            spanner_instance_admin_pb2.ListInstanceConfigsRequest(
-                parent=self.PATH, page_size=page_size, page_token=token
-            ),
+            ListInstanceConfigsRequest(parent=self.PATH, page_size=page_size),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
@@ -532,6 +558,7 @@ class TestClient(unittest.TestCase):
         self.assertIsNone(instance.configuration_name)
         self.assertEqual(instance.display_name, self.INSTANCE_ID)
         self.assertEqual(instance.node_count, DEFAULT_NODE_COUNT)
+        self.assertEqual(instance.labels, {})
         self.assertIs(instance._client, client)
 
     def test_instance_factory_explicit(self):
@@ -545,6 +572,7 @@ class TestClient(unittest.TestCase):
             self.CONFIGURATION_NAME,
             display_name=self.DISPLAY_NAME,
             node_count=self.NODE_COUNT,
+            labels=self.LABELS,
         )
 
         self.assertIsInstance(instance, Instance)
@@ -552,85 +580,84 @@ class TestClient(unittest.TestCase):
         self.assertEqual(instance.configuration_name, self.CONFIGURATION_NAME)
         self.assertEqual(instance.display_name, self.DISPLAY_NAME)
         self.assertEqual(instance.node_count, self.NODE_COUNT)
+        self.assertEqual(instance.labels, self.LABELS)
         self.assertIs(instance._client, client)
 
     def test_list_instances(self):
-        from google.cloud.spanner_admin_instance_v1.gapic import instance_admin_client
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2,
-        )
-        from google.cloud.spanner_v1.client import Instance
+        from google.cloud.spanner_admin_instance_v1 import InstanceAdminClient
+        from google.cloud.spanner_admin_instance_v1 import Instance as InstancePB
+        from google.cloud.spanner_admin_instance_v1 import ListInstancesRequest
+        from google.cloud.spanner_admin_instance_v1 import ListInstancesResponse
 
-        api = instance_admin_client.InstanceAdminClient(mock.Mock())
+        api = InstanceAdminClient(credentials=mock.Mock())
         credentials = _make_credentials()
         client = self._make_one(project=self.PROJECT, credentials=credentials)
         client._instance_admin_api = api
 
-        instance_pbs = spanner_instance_admin_pb2.ListInstancesResponse(
+        instance_pbs = ListInstancesResponse(
             instances=[
-                spanner_instance_admin_pb2.Instance(
+                InstancePB(
                     name=self.INSTANCE_NAME,
                     config=self.CONFIGURATION_NAME,
                     display_name=self.DISPLAY_NAME,
                     node_count=self.NODE_COUNT,
+                    processing_units=self.PROCESSING_UNITS,
                 )
             ]
         )
 
-        li_api = api._inner_api_calls["list_instances"] = mock.Mock(
-            return_value=instance_pbs
-        )
+        li_api = api._transport._wrapped_methods[
+            api._transport.list_instances
+        ] = mock.Mock(return_value=instance_pbs)
 
         response = client.list_instances()
         instances = list(response)
 
         instance = instances[0]
-        self.assertIsInstance(instance, Instance)
+        self.assertIsInstance(instance, InstancePB)
         self.assertEqual(instance.name, self.INSTANCE_NAME)
-        self.assertEqual(instance.configuration_name, self.CONFIGURATION_NAME)
+        self.assertEqual(instance.config, self.CONFIGURATION_NAME)
         self.assertEqual(instance.display_name, self.DISPLAY_NAME)
         self.assertEqual(instance.node_count, self.NODE_COUNT)
+        self.assertEqual(instance.processing_units, self.PROCESSING_UNITS)
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", client.project_name),
             ("x-goog-request-params", "parent={}".format(client.project_name)),
-        ]
+        )
         li_api.assert_called_once_with(
-            spanner_instance_admin_pb2.ListInstancesRequest(parent=self.PATH),
+            ListInstancesRequest(parent=self.PATH),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
 
     def test_list_instances_w_options(self):
-        from google.cloud.spanner_admin_instance_v1.gapic import instance_admin_client
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import InstanceAdminClient
+        from google.cloud.spanner_admin_instance_v1 import ListInstancesRequest
+        from google.cloud.spanner_admin_instance_v1 import ListInstancesResponse
 
-        api = instance_admin_client.InstanceAdminClient(mock.Mock())
+        api = InstanceAdminClient(credentials=mock.Mock())
         credentials = _make_credentials()
         client = self._make_one(project=self.PROJECT, credentials=credentials)
         client._instance_admin_api = api
 
-        instance_pbs = spanner_instance_admin_pb2.ListInstancesResponse(instances=[])
+        instance_pbs = ListInstancesResponse(instances=[])
 
-        li_api = api._inner_api_calls["list_instances"] = mock.Mock(
-            return_value=instance_pbs
-        )
+        li_api = api._transport._wrapped_methods[
+            api._transport.list_instances
+        ] = mock.Mock(return_value=instance_pbs)
 
-        token = "token"
         page_size = 42
-        list(client.list_instances(page_token=token, page_size=42))
+        filter_ = "name:instance"
+        list(client.list_instances(filter_=filter_, page_size=42))
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", client.project_name),
             ("x-goog-request-params", "parent={}".format(client.project_name)),
-        ]
+        )
         li_api.assert_called_once_with(
-            spanner_instance_admin_pb2.ListInstancesRequest(
-                parent=self.PATH, page_size=page_size, page_token=token
-            ),
+            ListInstancesRequest(parent=self.PATH, filter=filter_, page_size=page_size),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
