@@ -13,7 +13,6 @@
 # limitations under the License.
 
 import unittest
-
 import mock
 
 
@@ -27,6 +26,7 @@ class TestInstance(unittest.TestCase):
     LOCATION = "projects/" + PROJECT + "/locations/" + CONFIG_NAME
     DISPLAY_NAME = "display_name"
     NODE_COUNT = 5
+    PROCESSING_UNITS = 5000
     OP_ID = 8915
     OP_NAME = "operations/projects/%s/instances/%soperations/%d" % (
         PROJECT,
@@ -38,6 +38,8 @@ class TestInstance(unittest.TestCase):
     TIMEOUT_SECONDS = 1
     DATABASE_ID = "database_id"
     DATABASE_NAME = "%s/databases/%s" % (INSTANCE_NAME, DATABASE_ID)
+    LABELS = {"test": "true"}
+    FIELD_MASK = ["config", "display_name", "processing_units", "labels"]
 
     def _getTargetClass(self):
         from google.cloud.spanner_v1.instance import Instance
@@ -57,6 +59,7 @@ class TestInstance(unittest.TestCase):
         self.assertIs(instance.configuration_name, None)
         self.assertEqual(instance.node_count, DEFAULT_NODE_COUNT)
         self.assertEqual(instance.display_name, self.INSTANCE_ID)
+        self.assertEqual(instance.labels, {})
 
     def test_constructor_non_default(self):
         DISPLAY_NAME = "display_name"
@@ -68,12 +71,14 @@ class TestInstance(unittest.TestCase):
             configuration_name=self.CONFIG_NAME,
             node_count=self.NODE_COUNT,
             display_name=DISPLAY_NAME,
+            labels=self.LABELS,
         )
         self.assertEqual(instance.instance_id, self.INSTANCE_ID)
         self.assertIs(instance._client, client)
         self.assertEqual(instance.configuration_name, self.CONFIG_NAME)
         self.assertEqual(instance.node_count, self.NODE_COUNT)
         self.assertEqual(instance.display_name, DISPLAY_NAME)
+        self.assertEqual(instance.labels, self.LABELS)
 
     def test_copy(self):
         DISPLAY_NAME = "display_name"
@@ -92,12 +97,10 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance, new_instance)
 
     def test__update_from_pb_success(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         display_name = "display_name"
-        instance_pb = admin_v1_pb2.Instance(display_name=display_name)
+        instance_pb = Instance(display_name=display_name)
 
         instance = self._make_one(None, None, None, None)
         self.assertEqual(instance.display_name, None)
@@ -105,11 +108,9 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance.display_name, display_name)
 
     def test__update_from_pb_no_display_name(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
-        instance_pb = admin_v1_pb2.Instance()
+        instance_pb = Instance()
         instance = self._make_one(None, None, None, None)
         self.assertEqual(instance.display_name, None)
         with self.assertRaises(ValueError):
@@ -117,44 +118,39 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance.display_name, None)
 
     def test_from_pb_bad_instance_name(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         instance_name = "INCORRECT_FORMAT"
-        instance_pb = admin_v1_pb2.Instance(name=instance_name)
+        instance_pb = Instance(name=instance_name)
 
         klass = self._getTargetClass()
         with self.assertRaises(ValueError):
             klass.from_pb(instance_pb, None)
 
     def test_from_pb_project_mistmatch(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         ALT_PROJECT = "ALT_PROJECT"
         client = _Client(project=ALT_PROJECT)
 
         self.assertNotEqual(self.PROJECT, ALT_PROJECT)
 
-        instance_pb = admin_v1_pb2.Instance(name=self.INSTANCE_NAME)
+        instance_pb = Instance(name=self.INSTANCE_NAME)
 
         klass = self._getTargetClass()
         with self.assertRaises(ValueError):
             klass.from_pb(instance_pb, client)
 
     def test_from_pb_success(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         client = _Client(project=self.PROJECT)
 
-        instance_pb = admin_v1_pb2.Instance(
+        instance_pb = Instance(
             name=self.INSTANCE_NAME,
             config=self.CONFIG_NAME,
             display_name=self.INSTANCE_ID,
+            labels=self.LABELS,
         )
 
         klass = self._getTargetClass()
@@ -163,6 +159,7 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance._client, client)
         self.assertEqual(instance.instance_id, self.INSTANCE_ID)
         self.assertEqual(instance.configuration_name, self.CONFIG_NAME)
+        self.assertEqual(instance.labels, self.LABELS)
 
     def test_name_property(self):
         client = _Client(project=self.PROJECT)
@@ -170,6 +167,14 @@ class TestInstance(unittest.TestCase):
         instance = self._make_one(self.INSTANCE_ID, client, self.CONFIG_NAME)
         self.assertEqual(instance.name, self.INSTANCE_NAME)
 
+    def test_labels_property(self):
+        client = _Client(project=self.PROJECT)
+
+        instance = self._make_one(
+            self.INSTANCE_ID, client, self.CONFIG_NAME, labels=self.LABELS
+        )
+        self.assertEqual(instance.labels, self.LABELS)
+
     def test___eq__(self):
         client = object()
         instance1 = self._make_one(self.INSTANCE_ID, client, self.CONFIG_NAME)
@@ -226,7 +231,7 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance.name, self.INSTANCE_NAME)
         self.assertEqual(instance.config, self.CONFIG_NAME)
         self.assertEqual(instance.display_name, self.INSTANCE_ID)
-        self.assertEqual(instance.node_count, 1)
+        self.assertEqual(instance.processing_units, 1000)
         self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
 
     def test_create_success(self):
@@ -241,6 +246,7 @@ class TestInstance(unittest.TestCase):
             configuration_name=self.CONFIG_NAME,
             display_name=self.DISPLAY_NAME,
             node_count=self.NODE_COUNT,
+            labels=self.LABELS,
         )
 
         future = instance.create()
@@ -253,7 +259,37 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance.name, self.INSTANCE_NAME)
         self.assertEqual(instance.config, self.CONFIG_NAME)
         self.assertEqual(instance.display_name, self.DISPLAY_NAME)
-        self.assertEqual(instance.node_count, self.NODE_COUNT)
+        self.assertEqual(instance.processing_units, self.PROCESSING_UNITS)
+        self.assertEqual(instance.labels, self.LABELS)
+        self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
+
+    def test_create_with_processing_units(self):
+        op_future = _FauxOperationFuture()
+        client = _Client(self.PROJECT)
+        api = client.instance_admin_api = _FauxInstanceAdminAPI(
+            _create_instance_response=op_future
+        )
+        instance = self._make_one(
+            self.INSTANCE_ID,
+            client,
+            configuration_name=self.CONFIG_NAME,
+            display_name=self.DISPLAY_NAME,
+            processing_units=self.PROCESSING_UNITS,
+            labels=self.LABELS,
+        )
+
+        future = instance.create()
+
+        self.assertIs(future, op_future)
+
+        (parent, instance_id, instance, metadata) = api._created_instance
+        self.assertEqual(parent, self.PARENT)
+        self.assertEqual(instance_id, self.INSTANCE_ID)
+        self.assertEqual(instance.name, self.INSTANCE_NAME)
+        self.assertEqual(instance.config, self.CONFIG_NAME)
+        self.assertEqual(instance.display_name, self.DISPLAY_NAME)
+        self.assertEqual(instance.processing_units, self.PROCESSING_UNITS)
+        self.assertEqual(instance.labels, self.LABELS)
         self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
 
     def test_exists_instance_grpc_error(self):
@@ -281,12 +317,10 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
 
     def test_exists_success(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         client = _Client(self.PROJECT)
-        instance_pb = admin_v1_pb2.Instance(
+        instance_pb = Instance(
             name=self.INSTANCE_NAME,
             config=self.CONFIG_NAME,
             display_name=self.DISPLAY_NAME,
@@ -331,16 +365,15 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
 
     def test_reload_success(self):
-        from google.cloud.spanner_admin_instance_v1.proto import (
-            spanner_instance_admin_pb2 as admin_v1_pb2,
-        )
+        from google.cloud.spanner_admin_instance_v1 import Instance
 
         client = _Client(self.PROJECT)
-        instance_pb = admin_v1_pb2.Instance(
+        instance_pb = Instance(
             name=self.INSTANCE_NAME,
             config=self.CONFIG_NAME,
             display_name=self.DISPLAY_NAME,
             node_count=self.NODE_COUNT,
+            labels=self.LABELS,
         )
         api = client.instance_admin_api = _FauxInstanceAdminAPI(
             _get_instance_response=instance_pb
@@ -352,6 +385,7 @@ class TestInstance(unittest.TestCase):
         self.assertEqual(instance.configuration_name, self.CONFIG_NAME)
         self.assertEqual(instance.node_count, self.NODE_COUNT)
         self.assertEqual(instance.display_name, self.DISPLAY_NAME)
+        self.assertEqual(instance.labels, self.LABELS)
 
         name, metadata = api._got_instance
         self.assertEqual(name, self.INSTANCE_NAME)
@@ -385,7 +419,7 @@ class TestInstance(unittest.TestCase):
             instance.update()
 
         instance, field_mask, metadata = api._updated_instance
-        self.assertEqual(field_mask.paths, ["config", "display_name", "node_count"])
+        self.assertEqual(field_mask.paths, self.FIELD_MASK)
         self.assertEqual(instance.name, self.INSTANCE_NAME)
         self.assertEqual(instance.config, self.CONFIG_NAME)
         self.assertEqual(instance.display_name, self.INSTANCE_ID)
@@ -404,6 +438,7 @@ class TestInstance(unittest.TestCase):
             configuration_name=self.CONFIG_NAME,
             node_count=self.NODE_COUNT,
             display_name=self.DISPLAY_NAME,
+            labels=self.LABELS,
         )
 
         future = instance.update()
@@ -411,11 +446,42 @@ class TestInstance(unittest.TestCase):
         self.assertIs(future, op_future)
 
         instance, field_mask, metadata = api._updated_instance
-        self.assertEqual(field_mask.paths, ["config", "display_name", "node_count"])
+        self.assertEqual(field_mask.paths, self.FIELD_MASK)
         self.assertEqual(instance.name, self.INSTANCE_NAME)
         self.assertEqual(instance.config, self.CONFIG_NAME)
         self.assertEqual(instance.display_name, self.DISPLAY_NAME)
         self.assertEqual(instance.node_count, self.NODE_COUNT)
+        self.assertEqual(instance.labels, self.LABELS)
+        self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
+
+    def test_update_success_with_processing_units(self):
+        op_future = _FauxOperationFuture()
+        client = _Client(self.PROJECT)
+        api = client.instance_admin_api = _FauxInstanceAdminAPI(
+            _update_instance_response=op_future
+        )
+        instance = self._make_one(
+            self.INSTANCE_ID,
+            client,
+            configuration_name=self.CONFIG_NAME,
+            processing_units=self.PROCESSING_UNITS,
+            display_name=self.DISPLAY_NAME,
+            labels=self.LABELS,
+        )
+
+        future = instance.update()
+
+        self.assertIs(future, op_future)
+
+        instance, field_mask, metadata = api._updated_instance
+        self.assertEqual(
+            field_mask.paths, ["config", "display_name", "processing_units", "labels"]
+        )
+        self.assertEqual(instance.name, self.INSTANCE_NAME)
+        self.assertEqual(instance.config, self.CONFIG_NAME)
+        self.assertEqual(instance.display_name, self.DISPLAY_NAME)
+        self.assertEqual(instance.processing_units, self.PROCESSING_UNITS)
+        self.assertEqual(instance.labels, self.LABELS)
         self.assertEqual(metadata, [("google-cloud-resource-prefix", instance.name)])
 
     def test_delete_grpc_error(self):
@@ -474,20 +540,31 @@ class TestInstance(unittest.TestCase):
         self.assertIs(database._instance, instance)
         self.assertEqual(list(database.ddl_statements), [])
         self.assertIsInstance(database._pool, BurstyPool)
+        self.assertIsNone(database._logger)
         pool = database._pool
         self.assertIs(pool._database, database)
+        self.assertIsNone(database.database_role)
 
     def test_database_factory_explicit(self):
+        from logging import Logger
         from google.cloud.spanner_v1.database import Database
         from tests._fixtures import DDL_STATEMENTS
 
         client = _Client(self.PROJECT)
         instance = self._make_one(self.INSTANCE_ID, client, self.CONFIG_NAME)
         DATABASE_ID = "database-id"
+        DATABASE_ROLE = "dummy-role"
         pool = _Pool()
+        logger = mock.create_autospec(Logger, instance=True)
+        encryption_config = {"kms_key_name": "kms_key_name"}
 
         database = instance.database(
-            DATABASE_ID, ddl_statements=DDL_STATEMENTS, pool=pool
+            DATABASE_ID,
+            ddl_statements=DDL_STATEMENTS,
+            pool=pool,
+            logger=logger,
+            encryption_config=encryption_config,
+            database_role=DATABASE_ROLE,
         )
 
         self.assertIsInstance(database, Database)
@@ -495,85 +572,79 @@ class TestInstance(unittest.TestCase):
         self.assertIs(database._instance, instance)
         self.assertEqual(list(database.ddl_statements), DDL_STATEMENTS)
         self.assertIs(database._pool, pool)
+        self.assertIs(database._logger, logger)
         self.assertIs(pool._bound, database)
+        self.assertIs(database._encryption_config, encryption_config)
+        self.assertIs(database.database_role, DATABASE_ROLE)
 
     def test_list_databases(self):
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import (
-            spanner_database_admin_pb2,
-        )
-        from google.cloud.spanner_v1.database import Database
+        from google.cloud.spanner_admin_database_v1 import Database as DatabasePB
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListDatabasesRequest
+        from google.cloud.spanner_admin_database_v1 import ListDatabasesResponse
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
-        databases_pb = spanner_database_admin_pb2.ListDatabasesResponse(
+        databases_pb = ListDatabasesResponse(
             databases=[
-                spanner_database_admin_pb2.Database(
-                    name="{}/databases/aa".format(self.INSTANCE_NAME)
-                ),
-                spanner_database_admin_pb2.Database(
-                    name="{}/databases/bb".format(self.INSTANCE_NAME)
-                ),
+                DatabasePB(name="{}/databases/aa".format(self.INSTANCE_NAME)),
+                DatabasePB(name="{}/databases/bb".format(self.INSTANCE_NAME)),
             ]
         )
 
-        ld_api = api._inner_api_calls["list_databases"] = mock.Mock(
-            return_value=databases_pb
-        )
+        ld_api = api._transport._wrapped_methods[
+            api._transport.list_databases
+        ] = mock.Mock(return_value=databases_pb)
 
         response = instance.list_databases()
         databases = list(response)
 
-        self.assertIsInstance(databases[0], Database)
+        self.assertIsInstance(databases[0], DatabasePB)
         self.assertTrue(databases[0].name.endswith("/aa"))
         self.assertTrue(databases[1].name.endswith("/bb"))
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ld_api.assert_called_once_with(
-            spanner_database_admin_pb2.ListDatabasesRequest(parent=self.INSTANCE_NAME),
+            ListDatabasesRequest(parent=self.INSTANCE_NAME),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
 
     def test_list_databases_w_options(self):
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import (
-            spanner_database_admin_pb2,
-        )
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListDatabasesRequest
+        from google.cloud.spanner_admin_database_v1 import ListDatabasesResponse
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
-        databases_pb = spanner_database_admin_pb2.ListDatabasesResponse(databases=[])
+        databases_pb = ListDatabasesResponse(databases=[])
 
-        ld_api = api._inner_api_calls["list_databases"] = mock.Mock(
-            return_value=databases_pb
-        )
+        ld_api = api._transport._wrapped_methods[
+            api._transport.list_databases
+        ] = mock.Mock(return_value=databases_pb)
 
         page_size = 42
-        page_token = "token"
-        response = instance.list_databases(page_size=page_size, page_token=page_token)
+        response = instance.list_databases(page_size=page_size)
         databases = list(response)
 
         self.assertEqual(databases, [])
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ld_api.assert_called_once_with(
-            spanner_database_admin_pb2.ListDatabasesRequest(
-                parent=self.INSTANCE_NAME, page_size=page_size, page_token=page_token
-            ),
+            ListDatabasesRequest(parent=self.INSTANCE_NAME, page_size=page_size),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
@@ -598,15 +669,23 @@ class TestInstance(unittest.TestCase):
         import datetime
         from google.cloud._helpers import UTC
         from google.cloud.spanner_v1.backup import Backup
+        from google.cloud.spanner_admin_database_v1 import CreateBackupEncryptionConfig
 
         client = _Client(self.PROJECT)
         instance = self._make_one(self.INSTANCE_ID, client, self.CONFIG_NAME)
         BACKUP_ID = "backup-id"
         DATABASE_NAME = "database-name"
         timestamp = datetime.datetime.utcnow().replace(tzinfo=UTC)
+        encryption_config = CreateBackupEncryptionConfig(
+            encryption_type=CreateBackupEncryptionConfig.EncryptionType.CUSTOMER_MANAGED_ENCRYPTION,
+            kms_key_name="kms_key_name",
+        )
 
         backup = instance.backup(
-            BACKUP_ID, database=DATABASE_NAME, expire_time=timestamp
+            BACKUP_ID,
+            database=DATABASE_NAME,
+            expire_time=timestamp,
+            encryption_config=encryption_config,
         )
 
         self.assertIsInstance(backup, Backup)
@@ -614,78 +693,81 @@ class TestInstance(unittest.TestCase):
         self.assertIs(backup._instance, instance)
         self.assertEqual(backup._database, DATABASE_NAME)
         self.assertIs(backup._expire_time, timestamp)
+        self.assertEqual(backup._encryption_config, encryption_config)
 
     def test_list_backups_defaults(self):
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import backup_pb2
-        from google.cloud.spanner_v1.backup import Backup
+        from google.cloud.spanner_admin_database_v1 import Backup as BackupPB
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListBackupsRequest
+        from google.cloud.spanner_admin_database_v1 import ListBackupsResponse
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
-        backups_pb = backup_pb2.ListBackupsResponse(
+        backups_pb = ListBackupsResponse(
             backups=[
-                backup_pb2.Backup(name=instance.name + "/backups/op1"),
-                backup_pb2.Backup(name=instance.name + "/backups/op2"),
-                backup_pb2.Backup(name=instance.name + "/backups/op3"),
+                BackupPB(name=instance.name + "/backups/op1"),
+                BackupPB(name=instance.name + "/backups/op2"),
+                BackupPB(name=instance.name + "/backups/op3"),
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_backups"] = mock.Mock(
-            return_value=backups_pb
-        )
+        lbo_api = api._transport._wrapped_methods[
+            api._transport.list_backups
+        ] = mock.Mock(return_value=backups_pb)
 
         backups = instance.list_backups()
 
         for backup in backups:
-            self.assertIsInstance(backup, Backup)
+            self.assertIsInstance(backup, BackupPB)
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
-        ldo_api.assert_called_once_with(
-            backup_pb2.ListBackupsRequest(parent=self.INSTANCE_NAME),
+        )
+        lbo_api.assert_called_once_with(
+            ListBackupsRequest(parent=self.INSTANCE_NAME),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
 
     def test_list_backups_w_options(self):
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import backup_pb2
-        from google.cloud.spanner_v1.backup import Backup
+        from google.cloud.spanner_admin_database_v1 import Backup as BackupPB
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListBackupsRequest
+        from google.cloud.spanner_admin_database_v1 import ListBackupsResponse
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
-        backups_pb = backup_pb2.ListBackupsResponse(
+        backups_pb = ListBackupsResponse(
             backups=[
-                backup_pb2.Backup(name=instance.name + "/backups/op1"),
-                backup_pb2.Backup(name=instance.name + "/backups/op2"),
-                backup_pb2.Backup(name=instance.name + "/backups/op3"),
+                BackupPB(name=instance.name + "/backups/op1"),
+                BackupPB(name=instance.name + "/backups/op2"),
+                BackupPB(name=instance.name + "/backups/op3"),
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_backups"] = mock.Mock(
-            return_value=backups_pb
-        )
+        ldo_api = api._transport._wrapped_methods[
+            api._transport.list_backups
+        ] = mock.Mock(return_value=backups_pb)
 
         backups = instance.list_backups(filter_="filter", page_size=10)
 
         for backup in backups:
-            self.assertIsInstance(backup, Backup)
+            self.assertIsInstance(backup, BackupPB)
 
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ldo_api.assert_called_once_with(
-            backup_pb2.ListBackupsRequest(
+            ListBackupsRequest(
                 parent=self.INSTANCE_NAME, filter="filter", page_size=10
             ),
             metadata=expected_metadata,
@@ -695,113 +777,128 @@ class TestInstance(unittest.TestCase):
 
     def test_list_backup_operations_defaults(self):
         from google.api_core.operation import Operation
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import backup_pb2
+        from google.cloud.spanner_admin_database_v1 import CreateBackupMetadata
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListBackupOperationsRequest
+        from google.cloud.spanner_admin_database_v1 import ListBackupOperationsResponse
         from google.longrunning import operations_pb2
         from google.protobuf.any_pb2 import Any
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
         create_backup_metadata = Any()
-        create_backup_metadata.Pack(backup_pb2.CreateBackupMetadata())
+        create_backup_metadata.Pack(
+            CreateBackupMetadata.pb(
+                CreateBackupMetadata(name="backup", database="database")
+            )
+        )
 
-        operations_pb = backup_pb2.ListBackupOperationsResponse(
+        operations_pb = ListBackupOperationsResponse(
             operations=[
                 operations_pb2.Operation(name="op1", metadata=create_backup_metadata)
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_backup_operations"] = mock.Mock(
-            return_value=operations_pb
-        )
+        ldo_api = api._transport._wrapped_methods[
+            api._transport.list_backup_operations
+        ] = mock.Mock(return_value=operations_pb)
 
-        operations = instance.list_backup_operations()
+        ops = instance.list_backup_operations()
 
-        for op in operations:
-            self.assertIsInstance(op, Operation)
-
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ldo_api.assert_called_once_with(
-            backup_pb2.ListBackupOperationsRequest(parent=self.INSTANCE_NAME),
+            ListBackupOperationsRequest(parent=self.INSTANCE_NAME),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
+        self.assertTrue(all([type(op) is Operation for op in ops]))
 
     def test_list_backup_operations_w_options(self):
         from google.api_core.operation import Operation
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import backup_pb2
+        from google.cloud.spanner_admin_database_v1 import CreateBackupMetadata
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListBackupOperationsRequest
+        from google.cloud.spanner_admin_database_v1 import ListBackupOperationsResponse
         from google.longrunning import operations_pb2
         from google.protobuf.any_pb2 import Any
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
         create_backup_metadata = Any()
-        create_backup_metadata.Pack(backup_pb2.CreateBackupMetadata())
+        create_backup_metadata.Pack(
+            CreateBackupMetadata.pb(
+                CreateBackupMetadata(name="backup", database="database")
+            )
+        )
 
-        operations_pb = backup_pb2.ListBackupOperationsResponse(
+        operations_pb = ListBackupOperationsResponse(
             operations=[
                 operations_pb2.Operation(name="op1", metadata=create_backup_metadata)
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_backup_operations"] = mock.Mock(
-            return_value=operations_pb
-        )
+        ldo_api = api._transport._wrapped_methods[
+            api._transport.list_backup_operations
+        ] = mock.Mock(return_value=operations_pb)
 
-        operations = instance.list_backup_operations(filter_="filter", page_size=10)
+        ops = instance.list_backup_operations(filter_="filter", page_size=10)
 
-        for op in operations:
-            self.assertIsInstance(op, Operation)
-
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ldo_api.assert_called_once_with(
-            backup_pb2.ListBackupOperationsRequest(
+            ListBackupOperationsRequest(
                 parent=self.INSTANCE_NAME, filter="filter", page_size=10
             ),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
+        self.assertTrue(all([type(op) is Operation for op in ops]))
 
     def test_list_database_operations_defaults(self):
         from google.api_core.operation import Operation
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import (
-            spanner_database_admin_pb2,
+        from google.cloud.spanner_admin_database_v1 import CreateDatabaseMetadata
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListDatabaseOperationsRequest
+        from google.cloud.spanner_admin_database_v1 import (
+            ListDatabaseOperationsResponse,
+        )
+        from google.cloud.spanner_admin_database_v1 import (
+            OptimizeRestoredDatabaseMetadata,
         )
         from google.longrunning import operations_pb2
         from google.protobuf.any_pb2 import Any
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
         create_database_metadata = Any()
         create_database_metadata.Pack(
-            spanner_database_admin_pb2.CreateDatabaseMetadata()
+            CreateDatabaseMetadata.pb(CreateDatabaseMetadata(database="database"))
         )
 
         optimize_database_metadata = Any()
         optimize_database_metadata.Pack(
-            spanner_database_admin_pb2.OptimizeRestoredDatabaseMetadata()
+            OptimizeRestoredDatabaseMetadata.pb(
+                OptimizeRestoredDatabaseMetadata(name="database")
+            )
         )
 
-        databases_pb = spanner_database_admin_pb2.ListDatabaseOperationsResponse(
+        databases_pb = ListDatabaseOperationsResponse(
             operations=[
                 operations_pb2.Operation(name="op1", metadata=create_database_metadata),
                 operations_pb2.Operation(
@@ -810,53 +907,61 @@ class TestInstance(unittest.TestCase):
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_database_operations"] = mock.Mock(
-            return_value=databases_pb
-        )
+        ldo_api = api._transport._wrapped_methods[
+            api._transport.list_database_operations
+        ] = mock.Mock(return_value=databases_pb)
 
-        operations = instance.list_database_operations()
+        ops = instance.list_database_operations()
 
-        for op in operations:
-            self.assertIsInstance(op, Operation)
-
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ldo_api.assert_called_once_with(
-            spanner_database_admin_pb2.ListDatabaseOperationsRequest(
-                parent=self.INSTANCE_NAME
-            ),
+            ListDatabaseOperationsRequest(parent=self.INSTANCE_NAME),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
+        self.assertTrue(all([type(op) is Operation for op in ops]))
 
     def test_list_database_operations_w_options(self):
         from google.api_core.operation import Operation
-        from google.cloud.spanner_admin_database_v1.gapic import database_admin_client
-        from google.cloud.spanner_admin_database_v1.proto import (
-            spanner_database_admin_pb2,
+        from google.cloud.spanner_admin_database_v1 import DatabaseAdminClient
+        from google.cloud.spanner_admin_database_v1 import ListDatabaseOperationsRequest
+        from google.cloud.spanner_admin_database_v1 import (
+            ListDatabaseOperationsResponse,
         )
+        from google.cloud.spanner_admin_database_v1 import RestoreDatabaseMetadata
+        from google.cloud.spanner_admin_database_v1 import RestoreSourceType
+        from google.cloud.spanner_admin_database_v1 import UpdateDatabaseDdlMetadata
         from google.longrunning import operations_pb2
         from google.protobuf.any_pb2 import Any
 
-        api = database_admin_client.DatabaseAdminClient(mock.Mock())
+        api = DatabaseAdminClient(credentials=mock.Mock())
         client = _Client(self.PROJECT)
         client.database_admin_api = api
         instance = self._make_one(self.INSTANCE_ID, client)
 
         restore_database_metadata = Any()
         restore_database_metadata.Pack(
-            spanner_database_admin_pb2.RestoreDatabaseMetadata()
+            RestoreDatabaseMetadata.pb(
+                RestoreDatabaseMetadata(
+                    name="database", source_type=RestoreSourceType.BACKUP
+                )
+            )
         )
 
         update_database_metadata = Any()
         update_database_metadata.Pack(
-            spanner_database_admin_pb2.UpdateDatabaseDdlMetadata()
+            UpdateDatabaseDdlMetadata.pb(
+                UpdateDatabaseDdlMetadata(
+                    database="database", statements=["statements"]
+                )
+            )
         )
 
-        databases_pb = spanner_database_admin_pb2.ListDatabaseOperationsResponse(
+        databases_pb = ListDatabaseOperationsResponse(
             operations=[
                 operations_pb2.Operation(
                     name="op1", metadata=restore_database_metadata
@@ -865,31 +970,29 @@ class TestInstance(unittest.TestCase):
             ]
         )
 
-        ldo_api = api._inner_api_calls["list_database_operations"] = mock.Mock(
-            return_value=databases_pb
-        )
+        ldo_api = api._transport._wrapped_methods[
+            api._transport.list_database_operations
+        ] = mock.Mock(return_value=databases_pb)
 
-        operations = instance.list_database_operations(filter_="filter", page_size=10)
+        ops = instance.list_database_operations(filter_="filter", page_size=10)
 
-        for op in operations:
-            self.assertIsInstance(op, Operation)
-
-        expected_metadata = [
+        expected_metadata = (
             ("google-cloud-resource-prefix", instance.name),
             ("x-goog-request-params", "parent={}".format(instance.name)),
-        ]
+        )
         ldo_api.assert_called_once_with(
-            spanner_database_admin_pb2.ListDatabaseOperationsRequest(
+            ListDatabaseOperationsRequest(
                 parent=self.INSTANCE_NAME, filter="filter", page_size=10
             ),
             metadata=expected_metadata,
             retry=mock.ANY,
             timeout=mock.ANY,
         )
+        self.assertTrue(all([type(op) is Operation for op in ops]))
 
     def test_type_string_to_type_pb_hit(self):
-        from google.cloud.spanner_admin_database_v1.proto import (
-            spanner_database_admin_pb2,
+        from google.cloud.spanner_admin_database_v1 import (
+            OptimizeRestoredDatabaseMetadata,
         )
         from google.cloud.spanner_v1 import instance
 
@@ -897,7 +1000,7 @@ class TestInstance(unittest.TestCase):
         self.assertIn(type_string, instance._OPERATION_METADATA_TYPES)
         self.assertEqual(
             instance._type_string_to_type_pb(type_string),
-            spanner_database_admin_pb2.OptimizeRestoredDatabaseMetadata,
+            OptimizeRestoredDatabaseMetadata,
         )
 
     def test_type_string_to_type_pb_miss(self):
@@ -912,6 +1015,7 @@ class _Client(object):
         self.project = project
         self.project_name = "projects/" + self.project
         self.timeout_seconds = timeout_seconds
+        self.route_to_leader_enabled = True
 
     def copy(self):
         from copy import deepcopy
