@@ -44,12 +44,15 @@ class TestAbstractSessionPool(unittest.TestCase):
         pool = self._make_one()
         self.assertIsNone(pool._database)
         self.assertEqual(pool.labels, {})
+        self.assertIsNone(pool.database_role)
 
     def test_ctor_explicit(self):
         labels = {"foo": "bar"}
-        pool = self._make_one(labels=labels)
+        database_role = "dummy-role"
+        pool = self._make_one(labels=labels, database_role=database_role)
         self.assertIsNone(pool._database)
         self.assertEqual(pool.labels, labels)
+        self.assertEqual(pool.database_role, database_role)
 
     def test_bind_abstract(self):
         pool = self._make_one()
@@ -82,7 +85,7 @@ class TestAbstractSessionPool(unittest.TestCase):
         new_session = pool._new_session()
 
         self.assertIs(new_session, session)
-        database.session.assert_called_once_with()
+        database.session.assert_called_once_with(labels={}, database_role=None)
 
     def test__new_session_w_labels(self):
         labels = {"foo": "bar"}
@@ -94,7 +97,19 @@ class TestAbstractSessionPool(unittest.TestCase):
         new_session = pool._new_session()
 
         self.assertIs(new_session, session)
-        database.session.assert_called_once_with(labels=labels)
+        database.session.assert_called_once_with(labels=labels, database_role=None)
+
+    def test__new_session_w_database_role(self):
+        database_role = "dummy-role"
+        pool = self._make_one(database_role=database_role)
+        database = pool._database = _make_database("name")
+        session = _make_session()
+        database.session.return_value = session
+
+        new_session = pool._new_session()
+
+        self.assertIs(new_session, session)
+        database.session.assert_called_once_with(labels={}, database_role=database_role)
 
     def test_session_wo_kwargs(self):
         from google.cloud.spanner_v1.pool import SessionCheckout
@@ -133,26 +148,34 @@ class TestFixedSizePool(unittest.TestCase):
         self.assertEqual(pool.default_timeout, 10)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, {})
+        self.assertIsNone(pool.database_role)
 
     def test_ctor_explicit(self):
         labels = {"foo": "bar"}
-        pool = self._make_one(size=4, default_timeout=30, labels=labels)
+        database_role = "dummy-role"
+        pool = self._make_one(
+            size=4, default_timeout=30, labels=labels, database_role=database_role
+        )
         self.assertIsNone(pool._database)
         self.assertEqual(pool.size, 4)
         self.assertEqual(pool.default_timeout, 30)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, labels)
+        self.assertEqual(pool.database_role, database_role)
 
     def test_bind(self):
+        database_role = "dummy-role"
         pool = self._make_one()
         database = _Database("name")
         SESSIONS = [_Session(database)] * 10
+        database._database_role = database_role
         database._sessions.extend(SESSIONS)
 
         pool.bind(database)
 
         self.assertIs(pool._database, database)
         self.assertEqual(pool.size, 10)
+        self.assertEqual(pool.database_role, database_role)
         self.assertEqual(pool.default_timeout, 10)
         self.assertTrue(pool._sessions.full())
 
@@ -191,29 +214,29 @@ class TestFixedSizePool(unittest.TestCase):
         self.assertFalse(pool._sessions.full())
 
     def test_get_empty_default_timeout(self):
-        from six.moves.queue import Empty
+        import queue
 
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
 
-        with self.assertRaises(Empty):
+        with self.assertRaises(queue.Empty):
             pool.get()
 
-        self.assertEqual(queue._got, {"block": True, "timeout": 10})
+        self.assertEqual(session_queue._got, {"block": True, "timeout": 10})
 
     def test_get_empty_explicit_timeout(self):
-        from six.moves.queue import Empty
+        import queue
 
         pool = self._make_one(size=1, default_timeout=0.1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
 
-        with self.assertRaises(Empty):
+        with self.assertRaises(queue.Empty):
             pool.get(timeout=1)
 
-        self.assertEqual(queue._got, {"block": True, "timeout": 1})
+        self.assertEqual(session_queue._got, {"block": True, "timeout": 1})
 
     def test_put_full(self):
-        from six.moves.queue import Full
+        import queue
 
         pool = self._make_one(size=4)
         database = _Database("name")
@@ -221,7 +244,7 @@ class TestFixedSizePool(unittest.TestCase):
         database._sessions.extend(SESSIONS)
         pool.bind(database)
 
-        with self.assertRaises(Full):
+        with self.assertRaises(queue.Full):
             pool.put(_Session(database))
 
         self.assertTrue(pool._sessions.full())
@@ -272,14 +295,25 @@ class TestBurstyPool(unittest.TestCase):
         self.assertEqual(pool.target_size, 10)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, {})
+        self.assertIsNone(pool.database_role)
 
     def test_ctor_explicit(self):
         labels = {"foo": "bar"}
-        pool = self._make_one(target_size=4, labels=labels)
+        database_role = "dummy-role"
+        pool = self._make_one(target_size=4, labels=labels, database_role=database_role)
         self.assertIsNone(pool._database)
         self.assertEqual(pool.target_size, 4)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, labels)
+        self.assertEqual(pool.database_role, database_role)
+
+    def test_ctor_explicit_w_database_role_in_db(self):
+        database_role = "dummy-role"
+        pool = self._make_one()
+        database = pool._database = _Database("name")
+        database._database_role = database_role
+        pool.bind(database)
+        self.assertEqual(pool.database_role, database_role)
 
     def test_get_empty(self):
         pool = self._make_one()
@@ -392,11 +426,17 @@ class TestPingingPool(unittest.TestCase):
         self.assertEqual(pool._delta.seconds, 3000)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, {})
+        self.assertIsNone(pool.database_role)
 
     def test_ctor_explicit(self):
         labels = {"foo": "bar"}
+        database_role = "dummy-role"
         pool = self._make_one(
-            size=4, default_timeout=30, ping_interval=1800, labels=labels
+            size=4,
+            default_timeout=30,
+            ping_interval=1800,
+            labels=labels,
+            database_role=database_role,
         )
         self.assertIsNone(pool._database)
         self.assertEqual(pool.size, 4)
@@ -404,6 +444,17 @@ class TestPingingPool(unittest.TestCase):
         self.assertEqual(pool._delta.seconds, 1800)
         self.assertTrue(pool._sessions.empty())
         self.assertEqual(pool.labels, labels)
+        self.assertEqual(pool.database_role, database_role)
+
+    def test_ctor_explicit_w_database_role_in_db(self):
+        database_role = "dummy-role"
+        pool = self._make_one()
+        database = pool._database = _Database("name")
+        SESSIONS = [_Session(database)] * 10
+        database._sessions.extend(SESSIONS)
+        database._database_role = database_role
+        pool.bind(database)
+        self.assertEqual(pool.database_role, database_role)
 
     def test_bind(self):
         pool = self._make_one()
@@ -481,29 +532,29 @@ class TestPingingPool(unittest.TestCase):
         self.assertFalse(pool._sessions.full())
 
     def test_get_empty_default_timeout(self):
-        from six.moves.queue import Empty
+        import queue
 
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
 
-        with self.assertRaises(Empty):
+        with self.assertRaises(queue.Empty):
             pool.get()
 
-        self.assertEqual(queue._got, {"block": True, "timeout": 10})
+        self.assertEqual(session_queue._got, {"block": True, "timeout": 10})
 
     def test_get_empty_explicit_timeout(self):
-        from six.moves.queue import Empty
+        import queue
 
         pool = self._make_one(size=1, default_timeout=0.1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
 
-        with self.assertRaises(Empty):
+        with self.assertRaises(queue.Empty):
             pool.get(timeout=1)
 
-        self.assertEqual(queue._got, {"block": True, "timeout": 1})
+        self.assertEqual(session_queue._got, {"block": True, "timeout": 1})
 
     def test_put_full(self):
-        from six.moves.queue import Full
+        import queue
 
         pool = self._make_one(size=4)
         database = _Database("name")
@@ -511,7 +562,7 @@ class TestPingingPool(unittest.TestCase):
         database._sessions.extend(SESSIONS)
         pool.bind(database)
 
-        with self.assertRaises(Full):
+        with self.assertRaises(queue.Full):
             pool.put(_Session(database))
 
         self.assertTrue(pool._sessions.full())
@@ -522,7 +573,7 @@ class TestPingingPool(unittest.TestCase):
         from google.cloud.spanner_v1 import pool as MUT
 
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
 
         now = datetime.datetime.utcnow()
         database = _Database("name")
@@ -531,8 +582,8 @@ class TestPingingPool(unittest.TestCase):
         with _Monkey(MUT, _NOW=lambda: now):
             pool.put(session)
 
-        self.assertEqual(len(queue._items), 1)
-        ping_after, queued = queue._items[0]
+        self.assertEqual(len(session_queue._items), 1)
+        ping_after, queued = session_queue._items[0]
         self.assertEqual(ping_after, now + datetime.timedelta(seconds=3000))
         self.assertIs(queued, session)
 
@@ -624,11 +675,17 @@ class TestTransactionPingingPool(unittest.TestCase):
         self.assertTrue(pool._sessions.empty())
         self.assertTrue(pool._pending_sessions.empty())
         self.assertEqual(pool.labels, {})
+        self.assertIsNone(pool.database_role)
 
     def test_ctor_explicit(self):
         labels = {"foo": "bar"}
+        database_role = "dummy-role"
         pool = self._make_one(
-            size=4, default_timeout=30, ping_interval=1800, labels=labels
+            size=4,
+            default_timeout=30,
+            ping_interval=1800,
+            labels=labels,
+            database_role=database_role,
         )
         self.assertIsNone(pool._database)
         self.assertEqual(pool.size, 4)
@@ -637,6 +694,17 @@ class TestTransactionPingingPool(unittest.TestCase):
         self.assertTrue(pool._sessions.empty())
         self.assertTrue(pool._pending_sessions.empty())
         self.assertEqual(pool.labels, labels)
+        self.assertEqual(pool.database_role, database_role)
+
+    def test_ctor_explicit_w_database_role_in_db(self):
+        database_role = "dummy-role"
+        pool = self._make_one()
+        database = pool._database = _Database("name")
+        SESSIONS = [_Session(database)] * 10
+        database._sessions.extend(SESSIONS)
+        database._database_role = database_role
+        pool.bind(database)
+        self.assertEqual(pool.database_role, database_role)
 
     def test_bind(self):
         pool = self._make_one()
@@ -656,7 +724,7 @@ class TestTransactionPingingPool(unittest.TestCase):
         for session in SESSIONS:
             session.create.assert_not_called()
             txn = session._transaction
-            txn.begin.assert_called_once_with()
+            txn.begin.assert_not_called()
 
         self.assertTrue(pool._pending_sessions.empty())
 
@@ -685,12 +753,12 @@ class TestTransactionPingingPool(unittest.TestCase):
         for session in SESSIONS:
             session.create.assert_not_called()
             txn = session._transaction
-            txn.begin.assert_called_once_with()
+            txn.begin.assert_not_called()
 
         self.assertTrue(pool._pending_sessions.empty())
 
     def test_put_full(self):
-        from six.moves.queue import Full
+        import queue
 
         pool = self._make_one(size=4)
         database = _Database("name")
@@ -698,14 +766,14 @@ class TestTransactionPingingPool(unittest.TestCase):
         database._sessions.extend(SESSIONS)
         pool.bind(database)
 
-        with self.assertRaises(Full):
+        with self.assertRaises(queue.Full):
             pool.put(_Session(database))
 
         self.assertTrue(pool._sessions.full())
 
     def test_put_non_full_w_active_txn(self):
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
         pending = pool._pending_sessions = _Queue()
         database = _Database("name")
         session = _Session(database)
@@ -713,8 +781,8 @@ class TestTransactionPingingPool(unittest.TestCase):
 
         pool.put(session)
 
-        self.assertEqual(len(queue._items), 1)
-        _, queued = queue._items[0]
+        self.assertEqual(len(session_queue._items), 1)
+        _, queued = session_queue._items[0]
         self.assertIs(queued, session)
 
         self.assertEqual(len(pending._items), 0)
@@ -722,7 +790,7 @@ class TestTransactionPingingPool(unittest.TestCase):
 
     def test_put_non_full_w_committed_txn(self):
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
         pending = pool._pending_sessions = _Queue()
         database = _Database("name")
         session = _Session(database)
@@ -731,7 +799,7 @@ class TestTransactionPingingPool(unittest.TestCase):
 
         pool.put(session)
 
-        self.assertEqual(len(queue._items), 0)
+        self.assertEqual(len(session_queue._items), 0)
 
         self.assertEqual(len(pending._items), 1)
         self.assertIs(pending._items[0], session)
@@ -740,14 +808,14 @@ class TestTransactionPingingPool(unittest.TestCase):
 
     def test_put_non_full(self):
         pool = self._make_one(size=1)
-        queue = pool._sessions = _Queue()
+        session_queue = pool._sessions = _Queue()
         pending = pool._pending_sessions = _Queue()
         database = _Database("name")
         session = _Session(database)
 
         pool.put(session)
 
-        self.assertEqual(len(queue._items), 0)
+        self.assertEqual(len(session_queue._items), 0)
         self.assertEqual(len(pending._items), 1)
         self.assertIs(pending._items[0], session)
 
@@ -771,7 +839,7 @@ class TestTransactionPingingPool(unittest.TestCase):
         pool.begin_pending_transactions()  # no raise
 
         for txn in TRANSACTIONS:
-            txn.begin.assert_called_once_with()
+            txn.begin.assert_not_called()
 
         self.assertTrue(pending.empty())
 
@@ -794,10 +862,12 @@ class TestSessionCheckout(unittest.TestCase):
 
     def test_ctor_w_kwargs(self):
         pool = _Pool()
-        checkout = self._make_one(pool, foo="bar")
+        checkout = self._make_one(pool, foo="bar", database_role="dummy-role")
         self.assertIs(checkout._pool, pool)
         self.assertIsNone(checkout._session)
-        self.assertEqual(checkout._kwargs, {"foo": "bar"})
+        self.assertEqual(
+            checkout._kwargs, {"foo": "bar", "database_role": "dummy-role"}
+        )
 
     def test_context_manager_wo_kwargs(self):
         session = object()
@@ -885,24 +955,48 @@ class _Database(object):
     def __init__(self, name):
         self.name = name
         self._sessions = []
+        self._database_role = None
+        self.database_id = name
+        self._route_to_leader_enabled = True
 
-        def mock_batch_create_sessions(db, session_count=10, timeout=10, metadata=[]):
-            from google.cloud.spanner_v1.proto import spanner_pb2
+        def mock_batch_create_sessions(
+            request=None,
+            timeout=10,
+            metadata=[],
+            labels={},
+        ):
+            from google.cloud.spanner_v1 import BatchCreateSessionsResponse
+            from google.cloud.spanner_v1 import Session
 
-            response = spanner_pb2.BatchCreateSessionsResponse()
-            if session_count < 2:
-                response.session.add()
+            database_role = request.session_template.creator_role if request else None
+            if request.session_count < 2:
+                response = BatchCreateSessionsResponse(
+                    session=[Session(creator_role=database_role, labels=labels)]
+                )
             else:
-                response.session.add()
-                response.session.add()
+                response = BatchCreateSessionsResponse(
+                    session=[
+                        Session(creator_role=database_role, labels=labels),
+                        Session(creator_role=database_role, labels=labels),
+                    ]
+                )
             return response
 
-        from google.cloud.spanner_v1.gapic.spanner_client import SpannerClient
+        from google.cloud.spanner_v1 import SpannerClient
 
         self.spanner_api = mock.create_autospec(SpannerClient, instance=True)
         self.spanner_api.batch_create_sessions.side_effect = mock_batch_create_sessions
 
-    def session(self):
+    @property
+    def database_role(self):
+        """Database role used in sessions to connect to this database.
+
+        :rtype: str
+        :returns: an str with the name of the database role.
+        """
+        return self._database_role
+
+    def session(self, **kwargs):
         # always return first session in the list
         # to avoid reversing the order of putting
         # sessions into pool (important for order tests)
@@ -923,13 +1017,13 @@ class _Queue(object):
         return len(self._items) >= self._size
 
     def get(self, **kwargs):
-        from six.moves.queue import Empty
+        import queue
 
         self._got = kwargs
         try:
             return self._items.pop()
         except IndexError:
-            raise Empty()
+            raise queue.Empty()
 
     def put(self, item, **kwargs):
         self._put = kwargs
