@@ -13,10 +13,15 @@
 # limitations under the License.
 
 
-import unittest
-
 import mock
 
+from google.cloud.spanner_v1 import RequestOptions
+from google.cloud.spanner_v1 import Type
+from google.cloud.spanner_v1 import TypeCode
+from google.api_core.retry import Retry
+from google.api_core import gapic_v1
+
+from tests._helpers import OpenTelemetryBase, StatusCode
 
 TABLE_NAME = "citizens"
 COLUMNS = ["email", "first_name", "last_name", "age"]
@@ -33,10 +38,10 @@ INSERT INTO citizens(first_name, last_name, age)
 VALUES ("Phred", "Phlyntstone", @age)
 """
 PARAMS = {"age": 30}
-PARAM_TYPES = {"age": "INT64"}
+PARAM_TYPES = {"age": Type(code=TypeCode.INT64)}
 
 
-class TestTransaction(unittest.TestCase):
+class TestTransaction(OpenTelemetryBase):
 
     PROJECT_ID = "project-id"
     INSTANCE_ID = "instance-id"
@@ -46,6 +51,14 @@ class TestTransaction(unittest.TestCase):
     SESSION_ID = "session-id"
     SESSION_NAME = DATABASE_NAME + "/sessions/" + SESSION_ID
     TRANSACTION_ID = b"DEADBEEF"
+    TRANSACTION_TAG = "transaction-tag"
+
+    BASE_ATTRIBUTES = {
+        "db.type": "spanner",
+        "db.url": "spanner.googleapis.com",
+        "db.instance": "testing",
+        "net.host.name": "spanner.googleapis.com",
+    }
 
     def _getTargetClass(self):
         from google.cloud.spanner_v1.transaction import Transaction
@@ -58,11 +71,9 @@ class TestTransaction(unittest.TestCase):
         return transaction
 
     def _make_spanner_api(self):
-        import google.cloud.spanner_v1.gapic.spanner_client
+        from google.cloud.spanner_v1 import SpannerClient
 
-        return mock.create_autospec(
-            google.cloud.spanner_v1.gapic.spanner_client.SpannerClient, instance=True
-        )
+        return mock.create_autospec(SpannerClient, instance=True)
 
     def test_ctor_session_w_existing_txn(self):
         session = _Session()
@@ -80,12 +91,6 @@ class TestTransaction(unittest.TestCase):
         self.assertTrue(transaction._multi_use)
         self.assertEqual(transaction._execute_sql_count, 0)
 
-    def test__check_state_not_begun(self):
-        session = _Session()
-        transaction = self._make_one(session)
-        with self.assertRaises(ValueError):
-            transaction._check_state()
-
     def test__check_state_already_committed(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -122,6 +127,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.begin()
 
+        self.assertNoSpans()
+
     def test_begin_already_rolled_back(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -129,6 +136,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.begin()
 
+        self.assertNoSpans()
+
     def test_begin_already_committed(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -136,6 +145,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.begin()
 
+        self.assertNoSpans()
+
     def test_begin_w_other_error(self):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
@@ -146,11 +157,15 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(RuntimeError):
             transaction.begin()
 
-    def test_begin_ok(self):
-        from google.cloud.spanner_v1.proto.transaction_pb2 import (
-            Transaction as TransactionPB,
+        self.assertSpanAttributes(
+            "CloudSpanner.BeginTransaction",
+            status=StatusCode.ERROR,
+            attributes=TestTransaction.BASE_ATTRIBUTES,
         )
 
+    def test_begin_ok(self):
+        from google.cloud.spanner_v1 import Transaction as TransactionPB
+
         transaction_pb = TransactionPB(id=self.TRANSACTION_ID)
         database = _Database()
         api = database.spanner_api = _FauxSpannerAPI(
@@ -166,14 +181,51 @@ class TestTransaction(unittest.TestCase):
 
         session_id, txn_options, metadata = api._begun
         self.assertEqual(session_id, session.name)
-        self.assertTrue(txn_options.HasField("read_write"))
-        self.assertEqual(metadata, [("google-cloud-resource-prefix", database.name)])
+        self.assertTrue(type(txn_options).pb(txn_options).HasField("read_write"))
+        self.assertEqual(
+            metadata,
+            [
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
+        )
+
+        self.assertSpanAttributes(
+            "CloudSpanner.BeginTransaction", attributes=TestTransaction.BASE_ATTRIBUTES
+        )
+
+    def test_begin_w_retry(self):
+        from google.cloud.spanner_v1 import (
+            Transaction as TransactionPB,
+        )
+        from google.api_core.exceptions import InternalServerError
+
+        database = _Database()
+        api = database.spanner_api = self._make_spanner_api()
+        database.spanner_api.begin_transaction.side_effect = [
+            InternalServerError("Received unexpected EOS on DATA frame from server"),
+            TransactionPB(id=self.TRANSACTION_ID),
+        ]
+
+        session = _Session(database)
+        transaction = self._make_one(session)
+        transaction.begin()
+
+        self.assertEqual(api.begin_transaction.call_count, 2)
 
     def test_rollback_not_begun(self):
-        session = _Session()
+        database = _Database()
+        api = database.spanner_api = self._make_spanner_api()
+        session = _Session(database)
         transaction = self._make_one(session)
-        with self.assertRaises(ValueError):
-            transaction.rollback()
+
+        transaction.rollback()
+        self.assertTrue(transaction.rolled_back)
+
+        # Since there was no transaction to be rolled back, rollbacl rpc is not called.
+        api.rollback.assert_not_called()
+
+        self.assertNoSpans()
 
     def test_rollback_already_committed(self):
         session = _Session()
@@ -183,6 +235,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.rollback()
 
+        self.assertNoSpans()
+
     def test_rollback_already_rolled_back(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -191,6 +245,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.rollback()
 
+        self.assertNoSpans()
+
     def test_rollback_w_other_error(self):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
@@ -205,6 +261,12 @@ class TestTransaction(unittest.TestCase):
 
         self.assertFalse(transaction.rolled_back)
 
+        self.assertSpanAttributes(
+            "CloudSpanner.Rollback",
+            status=StatusCode.ERROR,
+            attributes=TestTransaction.BASE_ATTRIBUTES,
+        )
+
     def test_rollback_ok(self):
         from google.protobuf.empty_pb2 import Empty
 
@@ -224,7 +286,17 @@ class TestTransaction(unittest.TestCase):
         session_id, txn_id, metadata = api._rolled_back
         self.assertEqual(session_id, session.name)
         self.assertEqual(txn_id, self.TRANSACTION_ID)
-        self.assertEqual(metadata, [("google-cloud-resource-prefix", database.name)])
+        self.assertEqual(
+            metadata,
+            [
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
+        )
+
+        self.assertSpanAttributes(
+            "CloudSpanner.Rollback", attributes=TestTransaction.BASE_ATTRIBUTES
+        )
 
     def test_commit_not_begun(self):
         session = _Session()
@@ -232,6 +304,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.commit()
 
+        self.assertNoSpans()
+
     def test_commit_already_committed(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -240,6 +314,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.commit()
 
+        self.assertNoSpans()
+
     def test_commit_already_rolled_back(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -248,6 +324,8 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(ValueError):
             transaction.commit()
 
+        self.assertNoSpans()
+
     def test_commit_w_other_error(self):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
@@ -262,37 +340,80 @@ class TestTransaction(unittest.TestCase):
 
         self.assertIsNone(transaction.committed)
 
-    def _commit_helper(self, mutate=True):
+        self.assertSpanAttributes(
+            "CloudSpanner.Commit",
+            status=StatusCode.ERROR,
+            attributes=dict(TestTransaction.BASE_ATTRIBUTES, num_mutations=1),
+        )
+
+    def _commit_helper(
+        self, mutate=True, return_commit_stats=False, request_options=None
+    ):
         import datetime
-        from google.cloud.spanner_v1.proto.spanner_pb2 import CommitResponse
+        from google.cloud.spanner_v1 import CommitResponse
         from google.cloud.spanner_v1.keyset import KeySet
         from google.cloud._helpers import UTC
-        from google.cloud._helpers import _datetime_to_pb_timestamp
 
         now = datetime.datetime.utcnow().replace(tzinfo=UTC)
-        now_pb = _datetime_to_pb_timestamp(now)
         keys = [[0], [1], [2]]
         keyset = KeySet(keys=keys)
-        response = CommitResponse(commit_timestamp=now_pb)
+        response = CommitResponse(commit_timestamp=now)
+        if return_commit_stats:
+            response.commit_stats.mutation_count = 4
         database = _Database()
         api = database.spanner_api = _FauxSpannerAPI(_commit_response=response)
         session = _Session(database)
         transaction = self._make_one(session)
         transaction._transaction_id = self.TRANSACTION_ID
+        transaction.transaction_tag = self.TRANSACTION_TAG
 
         if mutate:
             transaction.delete(TABLE_NAME, keyset)
 
-        transaction.commit()
+        transaction.commit(
+            return_commit_stats=return_commit_stats, request_options=request_options
+        )
 
         self.assertEqual(transaction.committed, now)
         self.assertIsNone(session._transaction)
 
-        session_id, mutations, txn_id, metadata = api._committed
+        session_id, mutations, txn_id, actual_request_options, metadata = api._committed
+
+        if request_options is None:
+            expected_request_options = RequestOptions(
+                transaction_tag=self.TRANSACTION_TAG
+            )
+        elif type(request_options) is dict:
+            expected_request_options = RequestOptions(request_options)
+            expected_request_options.transaction_tag = self.TRANSACTION_TAG
+            expected_request_options.request_tag = None
+        else:
+            expected_request_options = request_options
+            expected_request_options.transaction_tag = self.TRANSACTION_TAG
+            expected_request_options.request_tag = None
+
         self.assertEqual(session_id, session.name)
         self.assertEqual(txn_id, self.TRANSACTION_ID)
         self.assertEqual(mutations, transaction._mutations)
-        self.assertEqual(metadata, [("google-cloud-resource-prefix", database.name)])
+        self.assertEqual(
+            metadata,
+            [
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
+        )
+        self.assertEqual(actual_request_options, expected_request_options)
+
+        if return_commit_stats:
+            self.assertEqual(transaction.commit_stats.mutation_count, 4)
+
+        self.assertSpanAttributes(
+            "CloudSpanner.Commit",
+            attributes=dict(
+                TestTransaction.BASE_ATTRIBUTES,
+                num_mutations=len(transaction._mutations),
+            ),
+        )
 
     def test_commit_no_mutations(self):
         self._commit_helper(mutate=False)
@@ -300,6 +421,37 @@ class TestTransaction(unittest.TestCase):
     def test_commit_w_mutations(self):
         self._commit_helper(mutate=True)
 
+    def test_commit_w_return_commit_stats(self):
+        self._commit_helper(return_commit_stats=True)
+
+    def test_commit_w_request_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+        )
+        self._commit_helper(request_options=request_options)
+
+    def test_commit_w_transaction_tag_ignored_success(self):
+        request_options = RequestOptions(
+            transaction_tag="tag-1-1",
+        )
+        self._commit_helper(request_options=request_options)
+
+    def test_commit_w_request_and_transaction_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+            transaction_tag="tag-1-1",
+        )
+        self._commit_helper(request_options=request_options)
+
+    def test_commit_w_request_and_transaction_tag_dictionary_success(self):
+        request_options = {"request_tag": "tag-1", "transaction_tag": "tag-1-1"}
+        self._commit_helper(request_options=request_options)
+
+    def test_commit_w_incorrect_tag_dictionary_error(self):
+        request_options = {"incorrect_tag": "tag-1-1"}
+        with self.assertRaises(ValueError):
+            self._commit_helper(request_options=request_options)
+
     def test__make_params_pb_w_params_wo_param_types(self):
         session = _Session()
         transaction = self._make_one(session)
@@ -343,24 +495,31 @@ class TestTransaction(unittest.TestCase):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
         session = _Session(database)
-        session = _Session()
         transaction = self._make_one(session)
         transaction._transaction_id = self.TRANSACTION_ID
 
         with self.assertRaises(ValueError):
             transaction.execute_update(DML_QUERY_WITH_PARAM, PARAMS)
 
-    def _execute_update_helper(self, count=0, query_options=None):
+    def _execute_update_helper(
+        self,
+        count=0,
+        query_options=None,
+        request_options=None,
+        retry=gapic_v1.method.DEFAULT,
+        timeout=gapic_v1.method.DEFAULT,
+    ):
         from google.protobuf.struct_pb2 import Struct
-        from google.cloud.spanner_v1.proto.result_set_pb2 import (
+        from google.cloud.spanner_v1 import (
             ResultSet,
             ResultSetStats,
         )
-        from google.cloud.spanner_v1.proto.transaction_pb2 import TransactionSelector
+        from google.cloud.spanner_v1 import TransactionSelector
         from google.cloud.spanner_v1._helpers import (
             _make_value_pb,
             _merge_query_options,
         )
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         MODE = 2  # PROFILE
         stats_pb = ResultSetStats(row_count_exact=1)
@@ -370,14 +529,23 @@ class TestTransaction(unittest.TestCase):
         session = _Session(database)
         transaction = self._make_one(session)
         transaction._transaction_id = self.TRANSACTION_ID
+        transaction.transaction_tag = self.TRANSACTION_TAG
         transaction._execute_sql_count = count
 
+        if request_options is None:
+            request_options = RequestOptions()
+        elif type(request_options) is dict:
+            request_options = RequestOptions(request_options)
+
         row_count = transaction.execute_update(
             DML_QUERY_WITH_PARAM,
             PARAMS,
             PARAM_TYPES,
             query_mode=MODE,
             query_options=query_options,
+            request_options=request_options,
+            retry=retry,
+            timeout=timeout,
         )
 
         self.assertEqual(row_count, 1)
@@ -392,17 +560,28 @@ class TestTransaction(unittest.TestCase):
             expected_query_options = _merge_query_options(
                 expected_query_options, query_options
             )
+        expected_request_options = request_options
+        expected_request_options.transaction_tag = self.TRANSACTION_TAG
 
-        api.execute_sql.assert_called_once_with(
-            self.SESSION_NAME,
-            DML_QUERY_WITH_PARAM,
+        expected_request = ExecuteSqlRequest(
+            session=self.SESSION_NAME,
+            sql=DML_QUERY_WITH_PARAM,
             transaction=expected_transaction,
             params=expected_params,
             param_types=PARAM_TYPES,
             query_mode=MODE,
             query_options=expected_query_options,
+            request_options=request_options,
             seqno=count,
-            metadata=[("google-cloud-resource-prefix", database.name)],
+        )
+        api.execute_sql.assert_called_once_with(
+            request=expected_request,
+            retry=retry,
+            timeout=timeout,
+            metadata=[
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
         )
 
         self.assertEqual(transaction._execute_sql_count, count + 1)
@@ -410,9 +589,46 @@ class TestTransaction(unittest.TestCase):
     def test_execute_update_new_transaction(self):
         self._execute_update_helper()
 
+    def test_execute_update_w_request_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+        )
+        self._execute_update_helper(request_options=request_options)
+
+    def test_execute_update_w_transaction_tag_success(self):
+        request_options = RequestOptions(
+            transaction_tag="tag-1-1",
+        )
+        self._execute_update_helper(request_options=request_options)
+
+    def test_execute_update_w_request_and_transaction_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+            transaction_tag="tag-1-1",
+        )
+        self._execute_update_helper(request_options=request_options)
+
+    def test_execute_update_w_request_and_transaction_tag_dictionary_success(self):
+        request_options = {"request_tag": "tag-1", "transaction_tag": "tag-1-1"}
+        self._execute_update_helper(request_options=request_options)
+
+    def test_execute_update_w_incorrect_tag_dictionary_error(self):
+        request_options = {"incorrect_tag": "tag-1-1"}
+        with self.assertRaises(ValueError):
+            self._execute_update_helper(request_options=request_options)
+
     def test_execute_update_w_count(self):
         self._execute_update_helper(count=1)
 
+    def test_execute_update_w_timeout_param(self):
+        self._execute_update_helper(timeout=2.0)
+
+    def test_execute_update_w_retry_param(self):
+        self._execute_update_helper(retry=Retry(deadline=60))
+
+    def test_execute_update_w_timeout_and_retry_params(self):
+        self._execute_update_helper(retry=Retry(deadline=60), timeout=2.0)
+
     def test_execute_update_error(self):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
@@ -427,12 +643,19 @@ class TestTransaction(unittest.TestCase):
         self.assertEqual(transaction._execute_sql_count, 1)
 
     def test_execute_update_w_query_options(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         self._execute_update_helper(
             query_options=ExecuteSqlRequest.QueryOptions(optimizer_version="3")
         )
 
+    def test_execute_update_w_request_options(self):
+        self._execute_update_helper(
+            request_options=RequestOptions(
+                priority=RequestOptions.Priority.PRIORITY_MEDIUM
+            )
+        )
+
     def test_batch_update_other_error(self):
         database = _Database()
         database.spanner_api = self._make_spanner_api()
@@ -444,18 +667,20 @@ class TestTransaction(unittest.TestCase):
         with self.assertRaises(RuntimeError):
             transaction.batch_update(statements=[DML_QUERY])
 
-    def _batch_update_helper(self, error_after=None, count=0):
+    def _batch_update_helper(self, error_after=None, count=0, request_options=None):
         from google.rpc.status_pb2 import Status
         from google.protobuf.struct_pb2 import Struct
-        from google.cloud.spanner_v1.proto.result_set_pb2 import ResultSet
-        from google.cloud.spanner_v1.proto.result_set_pb2 import ResultSetStats
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteBatchDmlResponse
-        from google.cloud.spanner_v1.proto.transaction_pb2 import TransactionSelector
+        from google.cloud.spanner_v1 import param_types
+        from google.cloud.spanner_v1 import ResultSet
+        from google.cloud.spanner_v1 import ResultSetStats
+        from google.cloud.spanner_v1 import ExecuteBatchDmlRequest
+        from google.cloud.spanner_v1 import ExecuteBatchDmlResponse
+        from google.cloud.spanner_v1 import TransactionSelector
         from google.cloud.spanner_v1._helpers import _make_value_pb
 
         insert_dml = "INSERT INTO table(pkey, desc) VALUES (%pkey, %desc)"
         insert_params = {"pkey": 12345, "desc": "DESCRIPTION"}
-        insert_param_types = {"pkey": "INT64", "desc": "STRING"}
+        insert_param_types = {"pkey": param_types.INT64, "desc": param_types.STRING}
         update_dml = 'UPDATE table SET desc = desc + "-amended"'
         delete_dml = "DELETE FROM table WHERE desc IS NULL"
 
@@ -487,9 +712,17 @@ class TestTransaction(unittest.TestCase):
         session = _Session(database)
         transaction = self._make_one(session)
         transaction._transaction_id = self.TRANSACTION_ID
+        transaction.transaction_tag = self.TRANSACTION_TAG
         transaction._execute_sql_count = count
 
-        status, row_counts = transaction.batch_update(dml_statements)
+        if request_options is None:
+            request_options = RequestOptions()
+        elif type(request_options) is dict:
+            request_options = RequestOptions(request_options)
+
+        status, row_counts = transaction.batch_update(
+            dml_statements, request_options=request_options
+        )
 
         self.assertEqual(status, expected_status)
         self.assertEqual(row_counts, expected_row_counts)
@@ -501,32 +734,76 @@ class TestTransaction(unittest.TestCase):
             }
         )
         expected_statements = [
-            {
-                "sql": insert_dml,
-                "params": expected_insert_params,
-                "param_types": insert_param_types,
-            },
-            {"sql": update_dml},
-            {"sql": delete_dml},
+            ExecuteBatchDmlRequest.Statement(
+                sql=insert_dml,
+                params=expected_insert_params,
+                param_types=insert_param_types,
+            ),
+            ExecuteBatchDmlRequest.Statement(sql=update_dml),
+            ExecuteBatchDmlRequest.Statement(sql=delete_dml),
         ]
+        expected_request_options = request_options
+        expected_request_options.transaction_tag = self.TRANSACTION_TAG
 
-        api.execute_batch_dml.assert_called_once_with(
+        expected_request = ExecuteBatchDmlRequest(
             session=self.SESSION_NAME,
             transaction=expected_transaction,
             statements=expected_statements,
             seqno=count,
-            metadata=[("google-cloud-resource-prefix", database.name)],
+            request_options=expected_request_options,
+        )
+        api.execute_batch_dml.assert_called_once_with(
+            request=expected_request,
+            metadata=[
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
         )
 
         self.assertEqual(transaction._execute_sql_count, count + 1)
 
     def test_batch_update_wo_errors(self):
-        self._batch_update_helper()
+        self._batch_update_helper(
+            request_options=RequestOptions(
+                priority=RequestOptions.Priority.PRIORITY_MEDIUM
+            ),
+        )
+
+    def test_batch_update_w_request_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+        )
+        self._batch_update_helper(request_options=request_options)
+
+    def test_batch_update_w_transaction_tag_success(self):
+        request_options = RequestOptions(
+            transaction_tag="tag-1-1",
+        )
+        self._batch_update_helper(request_options=request_options)
+
+    def test_batch_update_w_request_and_transaction_tag_success(self):
+        request_options = RequestOptions(
+            request_tag="tag-1",
+            transaction_tag="tag-1-1",
+        )
+        self._batch_update_helper(request_options=request_options)
+
+    def test_batch_update_w_request_and_transaction_tag_dictionary_success(self):
+        request_options = {"request_tag": "tag-1", "transaction_tag": "tag-1-1"}
+        self._batch_update_helper(request_options=request_options)
+
+    def test_batch_update_w_incorrect_tag_dictionary_error(self):
+        request_options = {"incorrect_tag": "tag-1-1"}
+        with self.assertRaises(ValueError):
+            self._batch_update_helper(request_options=request_options)
 
     def test_batch_update_w_errors(self):
         self._batch_update_helper(error_after=2, count=1)
 
     def test_batch_update_error(self):
+        from google.cloud.spanner_v1 import Type
+        from google.cloud.spanner_v1 import TypeCode
+
         database = _Database()
         api = database.spanner_api = self._make_spanner_api()
         api.execute_batch_dml.side_effect = RuntimeError()
@@ -536,7 +813,10 @@ class TestTransaction(unittest.TestCase):
 
         insert_dml = "INSERT INTO table(pkey, desc) VALUES (%pkey, %desc)"
         insert_params = {"pkey": 12345, "desc": "DESCRIPTION"}
-        insert_param_types = {"pkey": "INT64", "desc": "STRING"}
+        insert_param_types = {
+            "pkey": Type(code=TypeCode.INT64),
+            "desc": Type(code=TypeCode.STRING),
+        }
         update_dml = 'UPDATE table SET desc = desc + "-amended"'
         delete_dml = "DELETE FROM table WHERE desc IS NULL"
 
@@ -553,18 +833,13 @@ class TestTransaction(unittest.TestCase):
 
     def test_context_mgr_success(self):
         import datetime
-        from google.cloud.spanner_v1.proto.spanner_pb2 import CommitResponse
-        from google.cloud.spanner_v1.proto.transaction_pb2 import (
-            Transaction as TransactionPB,
-        )
+        from google.cloud.spanner_v1 import CommitResponse
+        from google.cloud.spanner_v1 import Transaction as TransactionPB
         from google.cloud._helpers import UTC
-        from google.cloud._helpers import _datetime_to_pb_timestamp
 
         transaction_pb = TransactionPB(id=self.TRANSACTION_ID)
-        database = _Database()
         now = datetime.datetime.utcnow().replace(tzinfo=UTC)
-        now_pb = _datetime_to_pb_timestamp(now)
-        response = CommitResponse(commit_timestamp=now_pb)
+        response = CommitResponse(commit_timestamp=now)
         database = _Database()
         api = database.spanner_api = _FauxSpannerAPI(
             _begin_transaction_response=transaction_pb, _commit_response=response
@@ -577,19 +852,23 @@ class TestTransaction(unittest.TestCase):
 
         self.assertEqual(transaction.committed, now)
 
-        session_id, mutations, txn_id, metadata = api._committed
+        session_id, mutations, txn_id, _, metadata = api._committed
         self.assertEqual(session_id, self.SESSION_NAME)
         self.assertEqual(txn_id, self.TRANSACTION_ID)
         self.assertEqual(mutations, transaction._mutations)
-        self.assertEqual(metadata, [("google-cloud-resource-prefix", database.name)])
+        self.assertEqual(
+            metadata,
+            [
+                ("google-cloud-resource-prefix", database.name),
+                ("x-goog-spanner-route-to-leader", "true"),
+            ],
+        )
 
     def test_context_mgr_failure(self):
         from google.protobuf.empty_pb2 import Empty
 
         empty_pb = Empty()
-        from google.cloud.spanner_v1.proto.transaction_pb2 import (
-            Transaction as TransactionPB,
-        )
+        from google.cloud.spanner_v1 import Transaction as TransactionPB
 
         transaction_pb = TransactionPB(id=self.TRANSACTION_ID)
         database = _Database()
@@ -605,20 +884,15 @@ class TestTransaction(unittest.TestCase):
                 raise Exception("bail out")
 
         self.assertEqual(transaction.committed, None)
+        # Rollback rpc will not be called as there is no transaction id to be rolled back, rolled_back flag will be marked as true.
         self.assertTrue(transaction.rolled_back)
         self.assertEqual(len(transaction._mutations), 1)
-
         self.assertEqual(api._committed, None)
 
-        session_id, txn_id, metadata = api._rolled_back
-        self.assertEqual(session_id, session.name)
-        self.assertEqual(txn_id, self.TRANSACTION_ID)
-        self.assertEqual(metadata, [("google-cloud-resource-prefix", database.name)])
-
 
 class _Client(object):
     def __init__(self):
-        from google.cloud.spanner_v1.proto.spanner_pb2 import ExecuteSqlRequest
+        from google.cloud.spanner_v1 import ExecuteSqlRequest
 
         self._query_options = ExecuteSqlRequest.QueryOptions(optimizer_version="1")
 
@@ -632,6 +906,7 @@ class _Database(object):
     def __init__(self):
         self.name = "testing"
         self._instance = _Instance()
+        self._route_to_leader_enabled = True
 
 
 class _Session(object):
@@ -650,22 +925,25 @@ class _FauxSpannerAPI(object):
     def __init__(self, **kwargs):
         self.__dict__.update(**kwargs)
 
-    def begin_transaction(self, session, options_, metadata=None):
-        self._begun = (session, options_, metadata)
+    def begin_transaction(self, session=None, options=None, metadata=None):
+        self._begun = (session, options, metadata)
         return self._begin_transaction_response
 
-    def rollback(self, session, transaction_id, metadata=None):
+    def rollback(self, session=None, transaction_id=None, metadata=None):
         self._rolled_back = (session, transaction_id, metadata)
         return self._rollback_response
 
     def commit(
         self,
-        session,
-        mutations,
-        transaction_id="",
-        single_use_transaction=None,
+        request=None,
         metadata=None,
     ):
-        assert single_use_transaction is None
-        self._committed = (session, mutations, transaction_id, metadata)
+        assert not request.single_use_transaction
+        self._committed = (
+            request.session,
+            request.mutations,
+            request.transaction_id,
+            request.request_options,
+            metadata,
+        )
         return self._commit_response
